@@ -46,16 +46,6 @@ static const string
 EVOLUTION_CALENDAR_PRODID("PRODID:-//ACME//NONSGML SyncEvolution//EN"),
 EVOLUTION_CALENDAR_VERSION("VERSION:2.0");
 
-class unrefECalObjectList {
- public:
-    /** free list of ECalChange instances */
-    static void unref(GList *pointer) {
-        if (pointer) {
-            e_cal_free_object_list(pointer);
-        }
-    }
-};
-
 bool EvolutionCalendarSource::LUIDs::containsLUID(const ItemID &id) const
 {
     const_iterator it = findUID(id.m_uid);
@@ -101,35 +91,41 @@ static int granularity()
     return secs;
 }
 
-EvolutionCalendarSource::EvolutionCalendarSource(ECalSourceType type,
+EvolutionCalendarSource::EvolutionCalendarSource(EvolutionCalendarSourceType type,
                                                  const SyncSourceParams &params) :
     EvolutionSyncSource(params, granularity()),
     m_type(type)
 {
     switch (m_type) {
-     case E_CAL_SOURCE_TYPE_EVENT:
+     case EVOLUTION_CAL_SOURCE_TYPE_EVENTS:
         SyncSourceLogging::init(InitList<std::string>("SUMMARY") + "LOCATION",
                                 ", ",
                                 m_operations);
         m_typeName = "calendar";
+#ifndef USE_ECAL_CLIENT
         m_newSystem = e_cal_new_system_calendar;
+#endif
         break;
-     case E_CAL_SOURCE_TYPE_TODO:
+     case EVOLUTION_CAL_SOURCE_TYPE_TASKS:
         SyncSourceLogging::init(InitList<std::string>("SUMMARY"),
                                 ", ",
                                 m_operations);
         m_typeName = "task list";
+#ifndef USE_ECAL_CLIENT
         m_newSystem = e_cal_new_system_tasks;
+#endif
         break;
-     case E_CAL_SOURCE_TYPE_JOURNAL:
+     case EVOLUTION_CAL_SOURCE_TYPE_MEMOS:
         SyncSourceLogging::init(InitList<std::string>("SUBJECT"),
                                 ", ",
                                 m_operations);
         m_typeName = "memo list";
+#ifndef USE_ECAL_CLIENT
         // This is not available in older Evolution versions.
         // A configure check could detect that, but as this isn't
         // important the functionality is simply disabled.
         m_newSystem = NULL /* e_cal_new_system_memos */;
+#endif
         break;
      default:
         SyncContext::throwError("internal error, invalid calendar type");
@@ -139,19 +135,26 @@ EvolutionCalendarSource::EvolutionCalendarSource(ECalSourceType type,
 
 SyncSource::Databases EvolutionCalendarSource::getDatabases()
 {
-    ESourceList *sources = NULL;
-    GError *gerror = NULL;
+    ESourceList *tmp = NULL;
+    GErrorCXX gerror;
     Databases result;
 
-    if (!e_cal_get_sources(&sources, m_type, &gerror)) {
+    if (
+#ifdef USE_ECAL_CLIENT
+        !e_cal_client_get_sources(&tmp, sourceType(), gerror)
+#else
+        !e_cal_get_sources(&tmp, sourceType(), gerror)
+#endif
+        ) {
         // ignore unspecific errors (like on Maemo with no support for memos)
         // and continue with empty list (perhaps defaults work)
         if (!gerror) {
-            sources = NULL;
+            tmp = NULL;
         } else {
             throwError("unable to access backend databases", gerror);
         }
     }
+    ESourceListCXX sources(tmp, false);
 
     bool first = true;
     for (GSList *g = sources ? e_source_list_peek_groups (sources) : NULL;
@@ -168,6 +171,15 @@ SyncSource::Databases EvolutionCalendarSource::getDatabases()
         }
     }
 
+#ifdef USE_ECAL_CLIENT
+    if (result.empty()) {
+        ECalClientCXX calendar = ECalClientCXX::steal(e_cal_client_new_system(sourceType(), NULL));
+        if (calendar) {
+          const char *uri = e_client_get_uri (E_CLIENT ((ECalClient*)calendar));
+          result.push_back(Database("<<system>>", uri ? uri : "<<unknown uri>>"));
+        }
+    }
+#else
     if (result.empty() && m_newSystem) {
         eptr<ECal, GObject> calendar(m_newSystem());
         if (calendar.get()) {
@@ -176,10 +188,41 @@ SyncSource::Databases EvolutionCalendarSource::getDatabases()
             result.push_back(Database("<<system>>", uri ? uri : "<<unknown uri>>"));
         }
     }
+#endif
 
     return result;
 }
 
+#ifdef USE_ECAL_CLIENT
+static void
+handle_error_cb (EClient */*client*/, const gchar *error_msg, gpointer user_data)
+{
+    EvolutionCalendarSource *that = static_cast<EvolutionCalendarSource *>(user_data);
+    SE_LOG_ERROR(that, NULL, "%s", error_msg);
+}
+
+static gboolean
+handle_authentication_cb (EClient */*client*/, ECredentials *credentials, gpointer user_data)
+{
+    EvolutionCalendarSource *that = static_cast<EvolutionCalendarSource *>(user_data);
+    std::string passwd = that->getPassword();
+    std::string prompt = e_credentials_peek(credentials, E_CREDENTIALS_KEY_PROMPT_TEXT);
+    std::string key = e_credentials_peek(credentials, E_CREDENTIALS_KEY_PROMPT_KEY);
+
+    SE_LOG_DEBUG(that, NULL, "authentication requested, prompt \"%s\", key \"%s\" => %s",
+                 prompt.c_str(), key.c_str(),
+                 !passwd.empty() ? "returning configured password" : "no password configured");
+
+    if (!passwd.empty()) {
+        e_credentials_set (credentials, E_CREDENTIALS_KEY_PASSWORD, passwd.c_str());
+        return true;
+    } else {
+        return false;
+    }
+}
+
+#else
+
 char *EvolutionCalendarSource::authenticate(const char *prompt,
                                             const char *key)
 {
@@ -191,18 +234,83 @@ char *EvolutionCalendarSource::authenticate(const char *prompt,
     return !passwd.empty() ? strdup(passwd.c_str()) : NULL;
 }
 
+#endif
+
 void EvolutionCalendarSource::open()
 {
-    ESourceList *sources;
-    GError *gerror = NULL;
+    ESourceList *tmp;
+    GErrorCXX gerror;
+    bool onlyIfExists = false; // always try to create address book, because even if there is
+                               // a source there's no guarantee that the actual database was
+                               // created already; the original logic below for only setting
+                               // this when explicitly requesting a new database
+                               // therefore failed in some cases
 
-    if (!e_cal_get_sources(&sources, m_type, &gerror)) {
+#ifdef USE_ECAL_CLIENT
+    if (!e_cal_client_get_sources (&tmp, sourceType(), gerror)) {
         throwError("unable to access backend databases", gerror);
     }
+    ESourceListCXX sources(tmp, false);
+
+    string id = getDatabaseID();    
+    ESource *source = findSource(sources, id);
+    bool created = false;
+
+    // Open twice. This solves an issue where Evolution's CalDAV
+    // backend only updates its local cache *after* a sync (= while
+    // closing the calendar?), instead of doing it *before* a sync (in
+    // e_cal_open()).
+    //
+    // This workaround is applied to *all* backends because there might
+    // be others with similar problems and for local storage it is
+    // a reasonably cheap operation (so no harm there).
+    for (int retries = 0; retries < 2; retries++) {
+        if (!source) {
+            // might have been special "<<system>>" or "<<default>>", try that and
+            // creating address book from file:// URI before giving up
+            if ((id.empty() || id == "<<system>>")) {
+                m_calendar = ECalClientCXX::steal(e_cal_client_new_system(sourceType(), gerror));
+            } else if (!id.compare(0, 7, "file://")) {
+                m_calendar = ECalClientCXX::steal(e_cal_client_new_from_uri(id.c_str(), sourceType(), gerror));
+            } else {
+                throwError(string("not found: '") + id + "'");
+            }
+            created = true;
+        } else {
+            m_calendar = ECalClientCXX::steal(e_cal_client_new(source, sourceType(), gerror));
+        }
+
+        if (gerror) {
+            throwError("create calendar", gerror);
+        }
+
+        // Listen for errors
+        g_signal_connect (m_calendar, "backend-error", G_CALLBACK (handle_error_cb), this); 
+
+        // Handle authentication requests from the backend
+        g_signal_connect (m_calendar, "authenticate", G_CALLBACK (handle_authentication_cb), this);
+    
+        if (!e_client_open_sync(E_CLIENT ((ECalClient*)m_calendar), onlyIfExists, NULL, gerror)) {
+            if (created) {
+                // opening newly created address books often failed, perhaps that also applies to calendars - try again
+                gerror.clear();
+                sleep(5);
+                if (!e_client_open_sync(E_CLIENT ((ECalClient*)m_calendar), onlyIfExists, NULL, gerror)) {
+                    throwError(string("opening ") + m_typeName , gerror);
+                }
+            } else {
+                throwError(string("opening ") + m_typeName , gerror);
+            }
+        }
+    }
+#else
+    if (!e_cal_get_sources(&tmp, sourceType(), gerror)) {
+        throwError("unable to access backend databases", gerror);
+    }
+    ESourceListCXX sources(tmp, false);
 
     string id = getDatabaseID();    
     ESource *source = findSource(sources, id);
-    bool onlyIfExists = true;
     bool created = false;
 
     // Open twice. This solves an issue where Evolution's CalDAV
@@ -220,32 +328,34 @@ void EvolutionCalendarSource::open()
             if ((id.empty() || id == "<<system>>") && m_newSystem) {
                 m_calendar.set(m_newSystem(), (string("system ") + m_typeName).c_str());
             } else if (!id.compare(0, 7, "file://")) {
-                m_calendar.set(e_cal_new_from_uri(id.c_str(), m_type), (string("creating ") + m_typeName).c_str());
+                m_calendar.set(e_cal_new_from_uri(id.c_str(), sourceType()), (string("creating ") + m_typeName).c_str());
             } else {
                 throwError(string("not found: '") + id + "'");
             }
             created = true;
             onlyIfExists = false;
         } else {
-            m_calendar.set(e_cal_new(source, m_type), m_typeName.c_str());
+            m_calendar.set(e_cal_new(source, sourceType()), m_typeName.c_str());
         }
 
         e_cal_set_auth_func(m_calendar, eCalAuthFunc, this);
     
-        if (!e_cal_open(m_calendar, onlyIfExists, &gerror)) {
+        if (!e_cal_open(m_calendar, onlyIfExists, gerror)) {
             if (created) {
                 // opening newly created address books often failed, perhaps that also applies to calendars - try again
-                g_clear_error(&gerror);
+                gerror.clear();
                 sleep(5);
-                if (!e_cal_open(m_calendar, onlyIfExists, &gerror)) {
-                    throwError(string("opening ") + m_typeName, gerror );
+                if (!e_cal_open(m_calendar, onlyIfExists, gerror)) {
+                    throwError(string("opening ") + m_typeName, gerror);
                 }
             } else {
-                throwError(string("opening ") + m_typeName, gerror );
+                throwError(string("opening ") + m_typeName, gerror);
             }
         }
     }
 
+#endif
+
     g_signal_connect_after(m_calendar,
                            "backend-died",
                            G_CALLBACK(SyncContext::fatalError),
@@ -261,17 +371,118 @@ bool EvolutionCalendarSource::isEmpty()
     return revisions.empty();
 }
 
+#ifdef USE_ECAL_CLIENT
+class ECalClientViewSyncHandler {
+  public:
+    ECalClientViewSyncHandler(ECalClientView *view, void (*processList)(const GSList *list, void *user_data), void *user_data): 
+        m_processList(processList), m_userData(user_data), m_view(view)
+    {}
+
+    bool processSync(GErrorCXX &gerror)
+    {
+        // Listen for view signals
+        g_signal_connect(m_view, "objects-added", G_CALLBACK(objectsAdded), this);
+        g_signal_connect(m_view, "complete", G_CALLBACK(completed), this);
+
+        // Start the view
+        e_cal_client_view_start (m_view, m_error);
+        if (m_error) {
+            std::swap(gerror, m_error);
+            return false;
+        }
+
+        // Async -> Sync
+        m_loop.run();
+        e_cal_client_view_stop (m_view, NULL);
+
+        if (m_error) {
+            std::swap(gerror, m_error);
+            return false;
+        } else {
+            return true;
+        }
+    }
+ 
+    static void objectsAdded(ECalClientView *ebookview,
+                             const GSList *objects,
+                             gpointer user_data) {
+        ECalClientViewSyncHandler *that = (ECalClientViewSyncHandler *)user_data;
+        that->m_processList(objects, that->m_userData);
+    }
+ 
+    static void completed(ECalClientView *ebookview,
+                          const GError *error,
+                          gpointer user_data) {
+        ECalClientViewSyncHandler *that = (ECalClientViewSyncHandler *)user_data;
+        that->m_error = error;
+        that->m_loop.quit();
+    }
+ 
+    public:
+      // Process list callback
+      void (*m_processList)(const GSList *list, void *user_data);
+      void *m_userData;
+      // Event loop for Async -> Sync
+      EvolutionAsync m_loop;
+
+    private:
+      // View watched
+      ECalClientView *m_view;
+
+      // Possible error while watching the view
+      GErrorCXX m_error;
+};
+
+static void list_revisions(const GSList *objects, void *user_data)
+{
+    EvolutionCalendarSource::RevisionMap_t *revisions = 
+        static_cast<EvolutionCalendarSource::RevisionMap_t *>(user_data);
+    const GSList *l;
+
+    for (l = objects; l; l = l->next) {
+        icalcomponent *icomp = (icalcomponent*)l->data;
+        EvolutionCalendarSource::ItemID id = EvolutionCalendarSource::getItemID(icomp);
+        string luid = id.getLUID();
+        string modTime = EvolutionCalendarSource::getItemModTime(icomp);
+
+        (*revisions)[luid] = modTime;
+    }
+}
+#endif
+
 void EvolutionCalendarSource::listAllItems(RevisionMap_t &revisions)
 {
-    GError *gerror = NULL;
+    GErrorCXX gerror;
+#ifdef USE_ECAL_CLIENT
+    ECalClientView *view;
+
+    if (!e_cal_client_get_view_sync (m_calendar, "#t", &view, NULL, gerror)) {
+        throwError( "getting the view" , gerror);
+    }
+    ECalClientViewCXX viewPtr = ECalClientViewCXX::steal(view);
+
+    // TODO: Optimization: use set fields_of_interest (UID / REV / LAST-MODIFIED)
+
+    ECalClientViewSyncHandler handler(viewPtr, list_revisions, &revisions);
+    if (!handler.processSync(gerror)) {
+        throwError("watching view", gerror);
+    }
+
+    // Update m_allLUIDs
+    m_allLUIDs.clear();
+    RevisionMap_t::iterator it;
+    for(it = revisions.begin(); it != revisions.end(); it++) {
+        m_allLUIDs.insertLUID(it->first);
+    }
+#else
     GList *nextItem;
 
     m_allLUIDs.clear();
     if (!e_cal_get_object_list_as_comp(m_calendar,
                                        "#t",
                                        &nextItem,
-                                       &gerror)) {
-        throwError( "reading all items", gerror );
+                                       gerror)) {
+        throwError("reading all items", gerror);
     }
     eptr<GList> listptr(nextItem);
     while (nextItem) {
@@ -284,6 +495,7 @@ void EvolutionCalendarSource::listAllItems(RevisionMap_t &revisions)
         revisions[luid] = modTime;
         nextItem = nextItem->next;
     }
+#endif
 }
 
 void EvolutionCalendarSource::close()
@@ -297,6 +509,32 @@ void EvolutionCalendarSource::readItem(const string &luid, std::string &item, bo
     item = retrieveItemAsString(id);
 }
 
+#ifdef USE_ECAL_CLIENT
+icaltimezone *
+my_tzlookup(const gchar *tzid,
+            gconstpointer ecalclient,
+            GCancellable *cancellable,
+            GError **error)
+{
+    icaltimezone *zone = NULL;
+    GError *local_error = NULL;
+
+    if (e_cal_client_get_timezone_sync((ECalClient *)ecalclient, tzid, &zone, cancellable, &local_error)) {
+        return zone;
+    } else if (local_error && local_error->domain == E_CAL_CLIENT_ERROR) {
+        // Ignore *all* E_CAL_CLIENT_ERROR errors, e_cal_client_get_timezone_sync() does
+        // not reliably return a specific code like E_CAL_CLIENT_ERROR_OBJECT_NOT_FOUND.
+        // See the 'e_cal_client_check_timezones() + e_cal_client_tzlookup() + Could not retrieve calendar time zone: Invalid object'
+        // mail thread.
+        g_clear_error (&local_error);
+    } else if (local_error) {
+        g_propagate_error (error, local_error);
+    }
+
+    return NULL;
+}
+#endif
+
 EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(const string &luid, const std::string &item, bool raw)
 {
     bool update = !luid.empty();
@@ -342,14 +580,25 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
         throwError(string("failure parsing ical") + data);
     }
 
-    GError *gerror = NULL;
+    GErrorCXX gerror;
 
     // fix up TZIDs
-    if (!e_cal_check_timezones(icomp,
+    if (
+#ifdef USE_ECAL_CLIENT
+        !e_cal_client_check_timezones(icomp,
+                                      NULL,
+                                      my_tzlookup,
+                                      (const void *)m_calendar.get(),
+                                      NULL,
+                                      gerror)
+#else
+        !e_cal_check_timezones(icomp,
                                NULL,
                                e_cal_tzlookup_ecal,
                                (const void *)m_calendar.get(),
-                               &gerror)) {
+                               gerror)
+#endif
+        ) {
         throwError(string("fixing timezones") + data,
                    gerror);
     }
@@ -362,13 +611,19 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
         eptr<icaltimezone> zone(icaltimezone_new(), "icaltimezone");
         icaltimezone_set_component(zone, tcomp);
 
-        GError *gerror = NULL;
+        GErrorCXX gerror;
         const char *tzid = icaltimezone_get_tzid(zone);
         if (!tzid || !tzid[0]) {
             // cannot add a VTIMEZONE without TZID
             SE_LOG_DEBUG(this, NULL, "skipping VTIMEZONE without TZID");
         } else {
-            gboolean success = e_cal_add_timezone(m_calendar, zone, &gerror);
+            gboolean success =
+#ifdef USE_ECAL_CLIENT
+                e_cal_client_add_timezone_sync(m_calendar, zone, NULL, gerror)
+#else
+                e_cal_add_timezone(m_calendar, zone, gerror)
+#endif
+                ;
             if (!success) {
                 throwError(string("error adding VTIMEZONE ") + tzid,
                            gerror);
@@ -433,13 +688,25 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
                 // problem by removing the children from the calendar,
                 // adding the parent, then updating it with the
                 // saved children.
+                //
+                // TODO: still necessary with e_cal_client API?
                 ICalComps_t children;
                 if (id.m_rid.empty()) {
                     children = removeEvents(id.m_uid, true);
                 }
 
                 // creating new objects works for normal events and detached occurrences alike
-                if(e_cal_create_object(m_calendar, subcomp, (char **)&uid, &gerror)) {
+                if (
+#ifdef USE_ECAL_CLIENT
+                    e_cal_client_create_object_sync(m_calendar, subcomp, (gchar **)&uid, 
+                                                    NULL, gerror)
+#else
+                    e_cal_create_object(m_calendar, subcomp, (gchar **)&uid, gerror)
+#endif
+                    ) {
+#ifdef USE_ECAL_CLIENT
+                    PlainGStr owner((gchar *)uid);
+#endif
                     // Evolution workaround: don't rely on uid being set if we already had
                     // one. In Evolution 2.12.1 it was set to garbage. The recurrence ID
                     // shouldn't have changed either.
@@ -454,9 +721,17 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
                 // Recreate any children removed earlier: when we get here,
                 // the parent exists and we must update it.
                 BOOST_FOREACH(boost::shared_ptr< eptr<icalcomponent> > &icalcomp, children) {
-                    if (!e_cal_modify_object(m_calendar, *icalcomp,
+                    if (
+#ifdef USE_ECAL_CLIENT
+                        !e_cal_client_modify_object_sync(m_calendar, *icalcomp,
+                                                         CALOBJ_MOD_THIS, NULL,
+                                                         gerror)
+#else
+                        !e_cal_modify_object(m_calendar, *icalcomp,
                                              CALOBJ_MOD_THIS,
-                                             &gerror)) {
+                                             gerror)
+#endif
+                        ) {
                         throwError(string("recreating item ") + newluid, gerror);
                     }
                 }
@@ -464,7 +739,9 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
         }
     }
 
-    if (update || state != ITEM_NEEDS_MERGE || detached) {
+    if (update ||
+        (state != ITEM_OKAY && state != ITEM_NEEDS_MERGE) ||
+        detached) {
         ItemID id(newluid);
         bool isParent = id.m_rid.empty();
 
@@ -519,32 +796,66 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
 
                 // Parent is gone, too, and needs to be recreated.
                 const char *uid = NULL;
-                if(!e_cal_create_object(m_calendar, subcomp, (char **)&uid, &gerror)) {
+                if (
+#ifdef USE_ECAL_CLIENT
+                    !e_cal_client_create_object_sync(m_calendar, subcomp, (char **)&uid, 
+                                                     NULL, gerror)
+#else
+                    !e_cal_create_object(m_calendar, subcomp, (char **)&uid, gerror)
+#endif
+                    ) {
                     throwError(string("creating updated item ") + luid, gerror);
                 }
+#ifdef USE_ECAL_CLIENT
+                PlainGStr owner((gchar *)uid);
+#endif
 
                 // Recreate any children removed earlier: when we get here,
                 // the parent exists and we must update it.
                 BOOST_FOREACH(boost::shared_ptr< eptr<icalcomponent> > &icalcomp, children) {
-                    if (!e_cal_modify_object(m_calendar, *icalcomp,
+                    if (
+#ifdef USE_ECAL_CLIENT
+                        !e_cal_client_modify_object_sync(m_calendar, *icalcomp,
+                                                         CALOBJ_MOD_THIS, NULL,
+                                                         gerror)
+#else
+                        !e_cal_modify_object(m_calendar, *icalcomp,
                                              CALOBJ_MOD_THIS,
-                                             &gerror)) {
+                                             gerror)
+#endif
+                        ) {
                         throwError(string("recreating item ") + luid, gerror);
                     }
                 }
             } else {
                 // no children, updating is simple
-                if (!e_cal_modify_object(m_calendar, subcomp,
+                if (
+#ifdef USE_ECAL_CLIENT
+                    !e_cal_client_modify_object_sync(m_calendar, subcomp,
+                                                     CALOBJ_MOD_ALL, NULL,
+                                                     gerror)
+#else
+                    !e_cal_modify_object(m_calendar, subcomp,
                                          CALOBJ_MOD_ALL,
-                                         &gerror)) {
+                                         gerror)
+#endif
+                    ) {
                     throwError(string("updating item ") + luid, gerror);
                 }
             }
         } else {
             // child event
-            if (!e_cal_modify_object(m_calendar, subcomp,
+            if (
+#ifdef USE_ECAL_CLIENT
+                !e_cal_client_modify_object_sync(m_calendar, subcomp,
+                                                 CALOBJ_MOD_THIS, NULL,
+                                                 gerror)
+#else
+                !e_cal_modify_object(m_calendar, subcomp,
                                      CALOBJ_MOD_THIS,
-                                     &gerror)) {
+                                     gerror)
+#endif
+                ) {
                 throwError(string("updating item ") + luid, gerror);
             }
         }
@@ -577,15 +888,22 @@ EvolutionCalendarSource::ICalComps_t EvolutionCalendarSource::removeEvents(const
     }
 
     // removes all events with that UID, including children
-    GError *gerror = NULL;
-    if(!e_cal_remove_object(m_calendar,
-                            uid.c_str(),
-                            &gerror)) {
-        if (gerror->domain == E_CALENDAR_ERROR &&
-            gerror->code == E_CALENDAR_STATUS_OBJECT_NOT_FOUND) {
+    GErrorCXX gerror;
+    if (
+#ifdef USE_ECAL_CLIENT
+        !e_cal_client_remove_object_sync(m_calendar,
+                                         uid.c_str(), NULL, CALOBJ_MOD_ALL,
+                                         NULL, gerror)
+
+#else
+        !e_cal_remove_object(m_calendar,
+                             uid.c_str(),
+                             gerror)
+#endif
+        ) {
+        if (IsCalObjNotFound(gerror)) {
             SE_LOG_DEBUG(this, NULL, "%s: request to delete non-existant item ignored",
                          uid.c_str());
-            g_clear_error(&gerror);
             if (!ignoreNotFound) {
                 throwError(STATUS_NOT_FOUND, string("delete item: ") + uid);
             }
@@ -599,7 +917,7 @@ EvolutionCalendarSource::ICalComps_t EvolutionCalendarSource::removeEvents(const
 
 void EvolutionCalendarSource::removeItem(const string &luid)
 {
-    GError *gerror = NULL;
+    GErrorCXX gerror;
     ItemID id(luid);
 
     if (id.m_rid.empty()) {
@@ -618,14 +936,32 @@ void EvolutionCalendarSource::removeItem(const string &luid)
             if (first) {
                 char *uid;
 
-                if (!e_cal_create_object(m_calendar, *icalcomp, &uid, &gerror)) {
+                if (
+#ifdef USE_ECAL_CLIENT
+                    !e_cal_client_create_object_sync(m_calendar, *icalcomp, &uid, 
+                                                     NULL, gerror)
+#else
+                    !e_cal_create_object(m_calendar, *icalcomp, &uid, gerror)
+#endif
+                    ) {
                     throwError(string("recreating first item ") + luid, gerror);
                 }
+#ifdef USE_ECAL_CLIENT
+                PlainGStr owner((gchar *)uid);
+#endif
                 first = false;
             } else {
-                if (!e_cal_modify_object(m_calendar, *icalcomp,
+                if (
+#ifdef USE_ECAL_CLIENT
+                    !e_cal_client_modify_object_sync(m_calendar, *icalcomp,
+                                                     CALOBJ_MOD_THIS, NULL,
+                                                     gerror)
+#else
+                    !e_cal_modify_object(m_calendar, *icalcomp,
                                          CALOBJ_MOD_THIS,
-                                         &gerror)) {
+                                         gerror)
+#endif
+                    ) {
                     throwError(string("recreating following item ") + luid, gerror);
                 }
             }
@@ -636,18 +972,26 @@ void EvolutionCalendarSource::removeItem(const string &luid)
         // therefore we have to check for existence first
         eptr<icalcomponent> item(retrieveItem(id));
         gboolean success = !item ? false :
+#ifdef USE_ECAL_CLIENT
+            // TODO: is this necessary?
+            e_cal_client_remove_object_sync(m_calendar,
+                                            id.m_uid.c_str(),
+                                            id.m_rid.c_str(),
+                                            CALOBJ_MOD_ONLY_THIS,
+                                            NULL,
+                                            gerror)
+#else
             e_cal_remove_object_with_mod(m_calendar,
                                          id.m_uid.c_str(),
                                          id.m_rid.c_str(),
                                          CALOBJ_MOD_THIS,
-                                         &gerror);
+                                         gerror)
+#endif
+            ;
         if (!item ||
-            (!success && gerror &&
-             gerror->domain == E_CALENDAR_ERROR &&
-             gerror->code == E_CALENDAR_STATUS_OBJECT_NOT_FOUND)) {
+            (!success && IsCalObjNotFound(gerror))) {
             SE_LOG_DEBUG(this, NULL, "%s: request to delete non-existant item",
                          luid.c_str());
-            g_clear_error(&gerror);
             throwError(STATUS_NOT_FOUND, string("delete item: ") + id.getLUID());
         } else if (!success) {
             throwError(string("deleting item " ) + luid, gerror);
@@ -679,16 +1023,26 @@ void EvolutionCalendarSource::removeItem(const string &luid)
 
 icalcomponent *EvolutionCalendarSource::retrieveItem(const ItemID &id)
 {
-    GError *gerror = NULL;
+    GErrorCXX gerror;
     icalcomponent *comp = NULL;
 
-    if (!e_cal_get_object(m_calendar,
+    if (
+#ifdef USE_ECAL_CLIENT
+        !e_cal_client_get_object_sync(m_calendar,
+                                      id.m_uid.c_str(),
+                                      !id.m_rid.empty() ? id.m_rid.c_str() : NULL,
+                                      &comp,
+                                      NULL,
+                                      gerror)
+#else
+        !e_cal_get_object(m_calendar,
                           id.m_uid.c_str(),
                           !id.m_rid.empty() ? id.m_rid.c_str() : NULL,
                           &comp,
-                          &gerror)) {
-        if (gerror && gerror->domain == E_CALENDAR_ERROR && gerror->code == E_CALENDAR_STATUS_OBJECT_NOT_FOUND) {
-            g_clear_error(&gerror);
+                          gerror)
+#endif
+        ) {
+        if (IsCalObjNotFound(gerror)) {
             throwError(STATUS_NOT_FOUND, string("retrieving item: ") + id.getLUID());
         } else {
             throwError(string("retrieving item: ") + id.getLUID(), gerror);
@@ -719,7 +1073,11 @@ string EvolutionCalendarSource::retrieveItemAsString(const ItemID &id)
     eptr<icalcomponent> comp(retrieveItem(id));
     eptr<char> icalstr;
 
+#ifdef USE_ECAL_CLIENT
+    icalstr = e_cal_client_get_component_as_string(m_calendar, comp);
+#else
     icalstr = e_cal_get_component_as_string(m_calendar, comp);
+#endif
 
     if (!icalstr) {
         // One reason why e_cal_get_component_as_string() can fail is
@@ -738,7 +1096,11 @@ string EvolutionCalendarSource::retrieveItemAsString(const ItemID &id)
         }
 
         // now try again
+#ifdef USE_ECAL_CLIENT
+        icalstr = e_cal_client_get_component_as_string(m_calendar, comp);
+#else
         icalstr = e_cal_get_component_as_string(m_calendar, comp);
+#endif
         if (!icalstr) {
             throwError(string("could not encode item as iCalendar: ") + id.getLUID());
         } else {
@@ -791,7 +1153,7 @@ std::string EvolutionCalendarSource::getDescription(const string &luid)
             descr += summary;
         }
         
-        if (m_type == E_CAL_SOURCE_TYPE_EVENT) {
+        if (m_type == EVOLUTION_CAL_SOURCE_TYPE_EVENTS) {
             const char *location = icalcomponent_get_location(comp);
             if (location && location[0]) {
                 if (!descr.empty()) {
@@ -801,7 +1163,7 @@ std::string EvolutionCalendarSource::getDescription(const string &luid)
             }
         }
 
-        if (m_type == E_CAL_SOURCE_TYPE_JOURNAL &&
+        if (m_type == EVOLUTION_CAL_SOURCE_TYPE_MEMOS &&
             descr.empty()) {
             // fallback to first line of body text
             icalproperty *desc = icalcomponent_get_first_property(comp, ICAL_DESCRIPTION_PROPERTY);
@@ -854,7 +1216,7 @@ EvolutionCalendarSource::ItemID EvolutionCalendarSource::getItemID(ECalComponent
 {
     icalcomponent *icomp = e_cal_component_get_icalcomponent(ecomp);
     if (!icomp) {
-        throwError("internal error in getItemID(): ECalComponent without icalcomp");
+        SE_THROW("internal error in getItemID(): ECalComponent without icalcomp");
     }
     return getItemID(icomp);
 }
@@ -885,13 +1247,18 @@ string EvolutionCalendarSource::getItemModTime(ECalComponent *ecomp)
 string EvolutionCalendarSource::getItemModTime(const ItemID &id)
 {
     eptr<icalcomponent> icomp(retrieveItem(id));
-    icalproperty *lastModified = icalcomponent_get_first_property(icomp, ICAL_LASTMODIFIED_PROPERTY);
-    if (!lastModified) {
+    return getItemModTime(icomp);
+}
+
+string EvolutionCalendarSource::getItemModTime(icalcomponent *icomp)
+{
+    icalproperty *modprop = icalcomponent_get_first_property(icomp, ICAL_LASTMODIFIED_PROPERTY);
+    if (!modprop) {
         return "";
-    } else {
-        struct icaltimetype modTime = icalproperty_get_lastmodified(lastModified);
-        return icalTime2Str(modTime);
     }
+    struct icaltimetype modTime = icalproperty_get_lastmodified(modprop);
+
+    return icalTime2Str(modTime);
 }
 
 string EvolutionCalendarSource::icalTime2Str(const icaltimetype &tt)
@@ -902,7 +1269,7 @@ string EvolutionCalendarSource::icalTime2Str(const icaltimetype &tt)
     } else {
         eptr<char> timestr(ical_strdup(icaltime_as_ical_string(tt)));
         if (!timestr) {
-            throwError("cannot convert to time string");
+            SE_THROW("cannot convert to time string");
         }
         return timestr.get();
     }
