@@ -41,24 +41,26 @@ using namespace std;
 #include <boost/foreach.hpp>
 
 #include <syncevo/declarations.h>
+
+void inline intrusive_ptr_add_ref(EBookQuery *ptr) { e_book_query_ref(ptr); }
+void inline intrusive_ptr_release(EBookQuery *ptr) { e_book_query_unref(ptr); }
+SE_BEGIN_CXX
+typedef boost::intrusive_ptr<EBookQuery> EBookQueryCXX;
+SE_END_CXX
+
 SE_BEGIN_CXX
 
-class unrefEBookChanges {
- public:
-    /** free list of EBookChange instances */
-    static void unref(GList *pointer) {
-        if (pointer) {
-            GList *next = pointer;
-            do {
-                EBookChange *ebc = (EBookChange *)next->data;
-                g_object_unref(ebc->contact);
-                g_free(next->data);
-                next = next->next;
-            } while (next);
-            g_list_free(pointer);
-        }
-    }
-};
+inline bool IsContactNotFound(const GError *gerror) {
+    return gerror &&
+#ifdef USE_EBOOK_CLIENT
+        gerror->domain == E_BOOK_CLIENT_ERROR &&
+        gerror->code == E_BOOK_CLIENT_ERROR_CONTACT_NOT_FOUND
+#else
+        gerror->domain == E_BOOK_ERROR &&
+        gerror->code == E_BOOK_ERROR_CONTACT_NOT_FOUND
+#endif
+        ;
+}
 
 
 const EvolutionContactSource::extensions EvolutionContactSource::m_vcardExtensions;
@@ -78,7 +80,11 @@ EvolutionSyncSource::Databases EvolutionContactSource::getDatabases()
 {
     ESourceList *sources = NULL;
 
+#ifdef USE_EBOOK_CLIENT
+    if (!e_book_client_get_sources(&sources, NULL)) {
+#else
     if (!e_book_get_addressbooks(&sources, NULL)) {
+#endif
         SyncContext::throwError("unable to access address books");
     }
 
@@ -120,23 +126,39 @@ EvolutionSyncSource::Databases EvolutionContactSource::getDatabases()
 
     // No results? Try system address book (workaround for embedded Evolution Dataserver).
     if (!result.size()) {
-        eptr<EBook, GObject> book;
-        GError *gerror = NULL;
+#ifdef USE_EBOOK_CLIENT
+        EBookClientCXX book;
         const char *name;
 
         name = "<<system>>";
-        book = e_book_new_system_addressbook (&gerror);
-        g_clear_error(&gerror);
+        book = EBookClientCXX::steal(e_book_client_new_system (NULL));
         if (!book) {
             name = "<<default>>";
-            book = e_book_new_default_addressbook (&gerror);
+            book = EBookClientCXX::steal(e_book_client_new_default (NULL));
+        }
+
+        if (book) {
+            const char *uri = e_client_get_uri (E_CLIENT ((EBookClient*)book));
+            result.push_back(Database(name, uri, true));
+        }
+#else
+        eptr<EBook, GObject> book;
+        GErrorCXX gerror;
+        const char *name;
+
+        name = "<<system>>";
+        book = e_book_new_system_addressbook (gerror);
+        gerror.clear();
+        if (!book) {
+            name = "<<default>>";
+            book = e_book_new_default_addressbook (gerror);
         }
-        g_clear_error(&gerror);
 
         if (book) {
             const char *uri = e_book_get_uri (book);
             result.push_back(Database(name, uri, true));
         }
+#endif
     } else {
         //  the first DB found is the default
         result[0].m_isDefault = true;
@@ -145,42 +167,125 @@ EvolutionSyncSource::Databases EvolutionContactSource::getDatabases()
     return result;
 }
 
+#ifdef USE_EBOOK_CLIENT
+static void
+handle_error_cb (EClient */*client*/, const gchar *error_msg, gpointer user_data)
+{
+    EvolutionContactSource *that = static_cast<EvolutionContactSource *>(user_data);
+    SE_LOG_ERROR(that, NULL, "%s", error_msg);
+}
+
+static gboolean
+handle_authentication_cb (EClient */*client*/, ECredentials *credentials, gpointer user_data)
+{
+    EvolutionContactSource *that = static_cast<EvolutionContactSource *>(user_data);
+    std::string user = that->getUser();
+    std::string passwd = that->getPassword();
+
+    if (!user.empty() || !passwd.empty()) {
+        e_credentials_set (credentials, E_CREDENTIALS_KEY_USERNAME, user.c_str());
+        e_credentials_set (credentials, E_CREDENTIALS_KEY_PASSWORD, passwd.c_str());
+        return true;
+    } else {
+        return false;
+    }
+}
+#endif
+
 void EvolutionContactSource::open()
 {
-    ESourceList *sources;
-    if (!e_book_get_addressbooks(&sources, NULL)) {
-        throwError("unable to access address books");
+    GErrorCXX gerror;
+    bool created = false;
+    bool onlyIfExists = false; // always try to create address book, because even if there is
+                               // a source there's no guarantee that the actual database was
+                               // created already; the original logic below for only setting
+                               // this when explicitly requesting a new address book
+                               // therefore failed in some cases
+#ifdef USE_EBOOK_CLIENT
+    ESourceList *tmp;
+    if (!e_book_client_get_sources(&tmp, gerror)) {
+        throwError("unable to access address books", gerror);
     }
-    
-    GError *gerror = NULL;
+    ESourceListCXX sources(tmp, false);
+
     string id = getDatabaseID();
     ESource *source = findSource(sources, id);
-    bool onlyIfExists = true;
-    bool created = false;
     if (!source) {
         // might have been special "<<system>>" or "<<default>>", try that and
         // creating address book from file:// URI before giving up
         if (id.empty() || id == "<<system>>") {
-            m_addressbook.set( e_book_new_system_addressbook (&gerror), "system address book" );
+            m_addressbook = EBookClientCXX::steal(e_book_client_new_system (gerror));
         } else if (id.empty() || id == "<<default>>") {
-            m_addressbook.set( e_book_new_default_addressbook (&gerror), "default address book" );
+            m_addressbook = EBookClientCXX::steal(e_book_client_new_default (gerror));
         } else if (boost::starts_with(id, "file://")) {
-            m_addressbook.set(e_book_new_from_uri(id.c_str(), &gerror), "creating address book");
+            m_addressbook = EBookClientCXX::steal(e_book_client_new_from_uri(id.c_str(), gerror));
         } else {
             throwError(string(getName()) + ": no such address book: '" + id + "'");
         }
         created = true;
-        onlyIfExists = false;
     } else {
-        m_addressbook.set( e_book_new( source, &gerror ), "address book" );
+        m_addressbook = EBookClientCXX::steal(e_book_client_new( source, gerror ));
     }
+
+    if (gerror) {
+        throwError("create addressbook", gerror);
+    }
+
+    // Listen for errors
+    g_signal_connect (m_addressbook, "backend-error", G_CALLBACK (handle_error_cb), this); 
+
+    // Handle authentication requests from the backend
+    g_signal_connect (m_addressbook, "authenticate", G_CALLBACK (handle_authentication_cb), this);
  
-    if (!e_book_open( m_addressbook, onlyIfExists, &gerror) ) {
+    // Open the address book
+    if (!e_client_open_sync( E_CLIENT ((EBookClient*)m_addressbook), onlyIfExists, NULL, gerror) ) {
         if (created) {
             // opening newly created address books often fails, try again once more
-            g_clear_error(&gerror);
+            gerror.clear();
             sleep(5);
-            if (!e_book_open(m_addressbook, onlyIfExists, &gerror)) {
+            if (!e_client_open_sync( E_CLIENT ((EBookClient*)m_addressbook), onlyIfExists, NULL, gerror)) {
+                throwError("opening address book", gerror);
+            }
+        } else {
+            throwError("opening address book", gerror);
+        }
+    }
+
+    g_signal_connect_after(m_addressbook,
+                           "backend-died",
+                           G_CALLBACK(SyncContext::fatalError),
+                           (void *)"Evolution Data Server has died unexpectedly, contacts no longer available.");
+#else
+    ESourceList *tmp;
+    if (!e_book_get_addressbooks(&tmp, gerror)) {
+        throwError("unable to access address books", gerror);
+    }
+    ESourceListCXX sources(tmp, false);
+
+    string id = getDatabaseID();
+    ESource *source = findSource(sources, id);
+    if (!source) {
+        // might have been special "<<system>>" or "<<default>>", try that and
+        // creating address book from file:// URI before giving up
+        if (id.empty() || id == "<<system>>") {
+            m_addressbook.set( e_book_new_system_addressbook (gerror), "system address book" );
+        } else if (id.empty() || id == "<<default>>") {
+            m_addressbook.set( e_book_new_default_addressbook (gerror), "default address book" );
+        } else if (boost::starts_with(id, "file://")) {
+            m_addressbook.set(e_book_new_from_uri(id.c_str(), gerror), "creating address book");
+        } else {
+            throwError(string(getName()) + ": no such address book: '" + id + "'");
+        }
+        created = true;
+    } else {
+        m_addressbook.set( e_book_new( source, gerror ), "address book" );
+    }
+ 
+    if (!e_book_open( m_addressbook, onlyIfExists, gerror) ) {
+        if (created) {
+            // opening newly created address books often fails, try again once more
+            sleep(5);
+            if (!e_book_open(m_addressbook, onlyIfExists, gerror)) {
                 throwError("opening address book", gerror);
             }
         } else {
@@ -195,8 +300,8 @@ void EvolutionContactSource::open()
         passwd = getPassword();
     if (!user.empty() || !passwd.empty()) {
         GList *authmethod;
-        if (!e_book_get_supported_auth_methods(m_addressbook, &authmethod, &gerror)) {
-            throwError("getting authentication methods", gerror );
+        if (!e_book_get_supported_auth_methods(m_addressbook, &authmethod, gerror)) {
+            throwError("getting authentication methods", gerror);
         }
         while (authmethod) {
             const char *method = (const char *)authmethod->data;
@@ -208,12 +313,11 @@ void EvolutionContactSource::open()
                                          user.c_str(),
                                          passwd.c_str(),
                                          method,
-                                         &gerror)) {
+                                         gerror)) {
                 SE_LOG_DEBUG(this, NULL, "authentication succeeded");
                 break;
             } else {
                 SE_LOG_ERROR(this, NULL, "authentication failed: %s", gerror->message);
-                g_clear_error(&gerror);
             }
             authmethod = authmethod->next;
         }
@@ -223,6 +327,7 @@ void EvolutionContactSource::open()
                            "backend-died",
                            G_CALLBACK(SyncContext::fatalError),
                            (void *)"Evolution Data Server has died unexpectedly, contacts no longer available.");
+#endif
 }
 
 bool EvolutionContactSource::isEmpty()
@@ -234,12 +339,130 @@ bool EvolutionContactSource::isEmpty()
     return revisions.empty();
 }
 
+#ifdef USE_EBOOK_CLIENT
+class EBookClientViewSyncHandler {
+    public:
+        EBookClientViewSyncHandler(EBookClientView *view, 
+                                   void (*processList)(const GSList *list, void *user_data), 
+                                   void *user_data): 
+            m_processList(processList), m_userData(user_data), m_view(view) {}
+
+        bool process(GErrorCXX &gerror) {
+                 // Listen for view signals
+            g_signal_connect(m_view, "objects-added", G_CALLBACK(contactsAdded), this);
+            g_signal_connect(m_view, "complete", G_CALLBACK(completed), this);
+
+            // Start the view
+            e_book_client_view_start (m_view, m_error);
+            if (m_error) {
+                std::swap(gerror, m_error);
+                return false;
+            }
+
+            // Async -> Sync
+            m_loop.run();
+            e_book_client_view_stop (m_view, NULL); 
+
+            if (m_error) {
+                std::swap(gerror, m_error);
+                return false;
+            } else {
+                return true;
+            }
+        }
+     
+        static void contactsAdded(EBookClientView *ebookview,
+                                  const GSList *contacts,
+                                  gpointer user_data) {
+            EBookClientViewSyncHandler *that = (EBookClientViewSyncHandler *)user_data;
+            that->m_processList(contacts, that->m_userData);
+        }
+ 
+        static void completed(EBookClientView *ebookview,
+                              const GError *error,
+                              gpointer user_data) {
+            EBookClientViewSyncHandler *that = (EBookClientViewSyncHandler *)user_data;
+            that->m_error = error;
+            that->m_loop.quit();
+        }
+
+    public:
+         // Process list callback
+         void (*m_processList)(const GSList *contacts, void *user_data);
+         void *m_userData;
+        // Event loop for Async -> Sync
+        EvolutionAsync m_loop;
+
+    private:
+        // View watched
+        EBookClientView *m_view;
+        // Possible error while watching the view
+        GErrorCXX m_error;
+};
+
+static void list_revisions(const GSList *contacts, void *user_data)
+{
+    EvolutionContactSource::RevisionMap_t *revisions = 
+        static_cast<EvolutionContactSource::RevisionMap_t *>(user_data);
+    const GSList *l;
+
+    for (l = contacts; l; l = l->next) {
+        EContact *contact = E_CONTACT(l->data);
+        if (!contact) {
+            SE_THROW("contact entry without data");
+        }
+        pair<string, string> revmapping;
+        const char *uid = (const char *)e_contact_get_const(contact,
+                                                            E_CONTACT_UID);
+        if (!uid || !uid[0]) {
+            SE_THROW("contact entry without UID");
+        }
+        revmapping.first = uid;
+        const char *rev = (const char *)e_contact_get_const(contact,
+                                                            E_CONTACT_REV);
+        if (!rev || !rev[0]) {
+            SE_THROW(string("contact entry without REV: ") + revmapping.first);
+        }
+        revmapping.second = rev;
+        revisions->insert(revmapping);
+    }
+}
+
+#endif
+
 void EvolutionContactSource::listAllItems(RevisionMap_t &revisions)
 {
-    GError *gerror = NULL;
+#ifdef USE_EBOOK_CLIENT
+    GErrorCXX gerror;
+    EBookClientView *view;
+
+    EBookQueryCXX allItemsQuery(e_book_query_any_field_contains(""), false);
+    PlainGStr sexp(e_book_query_to_string (allItemsQuery.get()));
+    
+    if (!e_book_client_get_view_sync(m_addressbook, sexp, &view, NULL, gerror)) {
+        throwError( "getting the view" , gerror);
+    }
+    EBookClientViewCXX viewPtr = EBookClientViewCXX::steal(view);
+
+    // Optimization: set fields_of_interest (UID / REV)
+    GListCXX<const char, GSList> interesting_field_list;
+    interesting_field_list.push_back(e_contact_field_name (E_CONTACT_UID));
+    interesting_field_list.push_back(e_contact_field_name (E_CONTACT_REV));
+    e_book_client_view_set_fields_of_interest (viewPtr, interesting_field_list, gerror);
+    if (gerror) {
+        SE_LOG_ERROR(this, NULL, "e_book_client_view_set_fields_of_interest: %s", (const char*)gerror);
+        gerror.clear();
+    }
+
+    EBookClientViewSyncHandler handler(viewPtr, list_revisions, &revisions);
+    if (!handler.process(gerror)) {
+        throwError("watching view", gerror);
+    }
+#else
+    GErrorCXX gerror;
     eptr<EBookQuery> allItemsQuery(e_book_query_any_field_contains(""), "query");
     GList *nextItem;
-    if (!e_book_get_contacts(m_addressbook, allItemsQuery, &nextItem, &gerror)) {
+    if (!e_book_get_contacts(m_addressbook, allItemsQuery, &nextItem, gerror)) {
         throwError( "reading all items", gerror );
     }
     eptr<GList> listptr(nextItem);
@@ -264,6 +487,7 @@ void EvolutionContactSource::listAllItems(RevisionMap_t &revisions)
         revisions.insert(revmapping);
         nextItem = nextItem->next;
     }
+#endif
 }
 
 void EvolutionContactSource::close()
@@ -274,13 +498,22 @@ void EvolutionContactSource::close()
 string EvolutionContactSource::getRevision(const string &luid)
 {
     EContact *contact;
-    GError *gerror = NULL;
-    if (!e_book_get_contact(m_addressbook,
+    GErrorCXX gerror;
+    if (
+#ifdef USE_EBOOK_CLIENT
+        !e_book_client_get_contact_sync(m_addressbook,
+                                        luid.c_str(),
+                                        &contact,
+                                        NULL,
+                                        gerror)
+#else
+        !e_book_get_contact(m_addressbook,
                             luid.c_str(),
                             &contact,
-                            &gerror)) {
-        if (gerror && gerror->domain == E_BOOK_ERROR && gerror->code == E_BOOK_ERROR_CONTACT_NOT_FOUND) {
-            g_clear_error(&gerror);
+                            gerror)
+#endif
+        ) {
+        if (IsContactNotFound(gerror)) {
             throwError(STATUS_NOT_FOUND, string("retrieving item: ") + luid);
         } else {
             throwError(string("reading contact ") + luid,
@@ -299,22 +532,48 @@ string EvolutionContactSource::getRevision(const string &luid)
 void EvolutionContactSource::readItem(const string &luid, std::string &item, bool raw)
 {
     EContact *contact;
-    GError *gerror = NULL;
-    if (!e_book_get_contact(m_addressbook,
+    GErrorCXX gerror;
+    if (
+#ifdef USE_EBOOK_CLIENT
+        !e_book_client_get_contact_sync(m_addressbook,
+                                        luid.c_str(),
+                                        &contact,
+                                        NULL,
+                                        gerror)
+#else
+        !e_book_get_contact(m_addressbook,
                             luid.c_str(),
                             &contact,
-                            &gerror)) {
-        if (gerror &&
-            gerror->domain == E_BOOK_ERROR &&
-            gerror->code == E_BOOK_ERROR_CONTACT_NOT_FOUND) {
-            g_clear_error(&gerror);
+                            gerror)
+#endif
+        ) {
+        if (IsContactNotFound(gerror)) {
             throwError(STATUS_NOT_FOUND, string("reading contact: ") + luid);
         } else {
             throwError(string("reading contact ") + luid,
                        gerror);
         }
     }
+
     eptr<EContact, GObject> contactptr(contact, "contact");
+
+    // Inline PHOTO data if exporting, leave VALUE=uri references unchanged
+    // when processing inside engine (will be inlined by engine as needed).
+    // The function for doing the inlining was added in EDS 3.4.
+    // In compatibility mode, we must check the function pointer for non-NULL.
+    // In direct call mode, the existence check is done by configure.
+    if (raw
+#ifdef EVOLUTION_COMPATIBILITY
+        && e_contact_inline_local_photos
+#endif
+        ) {
+#if defined(EVOLUTION_COMPATIBILITY) || defined(HAVE_E_CONTACT_INLINE_LOCAL_PHOTOS)
+        if (!e_contact_inline_local_photos(contactptr, gerror)) {
+            throwError(string("inlining PHOTO file data in ") + luid, gerror);
+        }
+#endif
+    }
+
     eptr<char> vcardstr(e_vcard_to_string(&contactptr->parent,
                                           EVC_FORMAT_VCARD_30));
     if (!vcardstr) {
@@ -329,14 +588,31 @@ EvolutionContactSource::insertItem(const string &uid, const std::string &item, b
 {
     eptr<EContact, GObject> contact(e_contact_new_from_vcard(item.c_str()));
     if (contact) {
-        GError *gerror = NULL;
         e_contact_set(contact, E_CONTACT_UID,
                       uid.empty() ?
                       NULL :
                       const_cast<char *>(uid.c_str()));
+        GErrorCXX gerror;
+#ifdef USE_EBOOK_CLIENT
+        if (uid.empty()) {
+            gchar* newuid;
+            if (!e_book_client_add_contact_sync(m_addressbook, contact, &newuid, NULL, gerror)) {
+                throwError("add new contact", gerror);
+            }
+            PlainGStr newuidPtr(newuid);
+            string newrev = getRevision(newuid);
+            return InsertItemResult(newuid, newrev, ITEM_OKAY);
+        } else {
+            if (!e_book_client_modify_contact_sync(m_addressbook, contact, NULL, gerror)) {
+                throwError("updating contact "+ uid, gerror);
+            }
+            string newrev = getRevision(uid);
+            return InsertItemResult(uid, newrev, ITEM_OKAY);
+        }
+#else
         if (uid.empty() ?
-            e_book_add_contact(m_addressbook, contact, &gerror) :
-            e_book_commit_contact(m_addressbook, contact, &gerror)) {
+            e_book_add_contact(m_addressbook, contact, gerror) :
+            e_book_commit_contact(m_addressbook, contact, gerror)) {
             const char *newuid = (const char *)e_contact_get_const(contact, E_CONTACT_UID);
             if (!newuid) {
                 throwError("no UID for contact");
@@ -349,6 +625,7 @@ EvolutionContactSource::insertItem(const string &uid, const std::string &item, b
                        string("updating contact ") + uid,
                        gerror);
         }
+#endif
     } else {
         throwError(string("failure parsing vcard " ) + item);
     }
@@ -358,16 +635,19 @@ EvolutionContactSource::insertItem(const string &uid, const std::string &item, b
 
 void EvolutionContactSource::removeItem(const string &uid)
 {
-    GError *gerror = NULL;
-    if (!e_book_remove_contact(m_addressbook, uid.c_str(), &gerror)) {
-        if (gerror &&
-            gerror->domain == E_BOOK_ERROR &&
-            gerror->code == E_BOOK_ERROR_CONTACT_NOT_FOUND) {
-            g_clear_error(&gerror);
+    GErrorCXX gerror;
+    if (
+#ifdef USE_EBOOK_CLIENT
+        !e_book_client_remove_contact_by_uid_sync(m_addressbook, uid.c_str(), NULL, gerror)
+#else
+        !e_book_remove_contact(m_addressbook, uid.c_str(), gerror)
+#endif
+        ) {
+        if (IsContactNotFound(gerror)) {
             throwError(STATUS_NOT_FOUND, string("deleting contact: ") + uid);
         } else {
             throwError( string( "deleting contact " ) + uid,
-                        gerror );
+                        gerror);
         }
     }
 }
@@ -376,11 +656,21 @@ std::string EvolutionContactSource::getDescription(const string &luid)
 {
     try {
         EContact *contact;
-        GError *gerror = NULL;
-        if (!e_book_get_contact(m_addressbook,
+        GErrorCXX gerror;
+        if (
+#ifdef USE_EBOOK_CLIENT
+            !e_book_client_get_contact_sync(m_addressbook,
+                                            luid.c_str(),
+                                            &contact,
+                                            NULL,
+                                            gerror)
+#else
+            !e_book_get_contact(m_addressbook,
                                 luid.c_str(),
                                 &contact,
-                                &gerror)) {
+                                gerror)
+#endif
+            ) {
             throwError(string("reading contact ") + luid,
                        gerror);
         }
