@@ -34,7 +34,7 @@ static SyncSource *createSource(const SyncSourceParams &params)
 #ifndef ENABLE_FILE
     // tell SyncEvolution if the user wanted to use a disabled sync source,
     // otherwise let it continue searching
-    return isMe ? RegisterSyncSource::InactiveSource : NULL;
+    return isMe ? RegisterSyncSource::InactiveSource(params) : NULL;
 #else
     // Also recognize one of the standard types?
     // Not in the FileSyncSource!
