@@ -53,7 +53,7 @@ MaemoCalendarSource::MaemoCalendarSource(int EntryType, int EntryFormat,
     }
 }
 
-std::string MaemoCalendarSource::getMimeType() const
+const char *MaemoCalendarSource::getMimeType() const
 {
     switch (entry_format) {
     case -1: return "text/plain";
@@ -63,7 +63,7 @@ std::string MaemoCalendarSource::getMimeType() const
     }
 }
 
-std::string MaemoCalendarSource::getMimeVersion() const
+const char *MaemoCalendarSource::getMimeVersion() const
 {
     switch (entry_format) {
     case -1: return "1.0";
@@ -113,13 +113,10 @@ void MaemoCalendarSource::open()
 
 bool MaemoCalendarSource::isEmpty()
 {
-    int id = cal->getCalendarId(), err;
-    switch (entry_type) {
-    case EVENT:   return !mc->getEventCount(id, err);
-    case TODO:    return !mc->getTodoCount(id, err);
-    case JOURNAL: return !mc->getNoteCount(id, err);
-    default:      return true;
-    }
+    // TODO: provide a real implementation. Always returning false
+    // here disables the "allow slow sync when no local data" heuristic
+    // for preventSlowSync=1.
+    return false;
 }
 
 void MaemoCalendarSource::close()
@@ -204,7 +201,7 @@ void MaemoCalendarSource::readItem(const string &uid, std::string &item, bool ra
         throwError(string("retrieving item: ") + uid);
     }
     if (entry_format == -1) {
-        item = c->getSummary();
+        item = c->getDescription();
         err = CALENDAR_OPERATION_SUCCESSFUL;
     } else {
         item = conv->localToIcalVcal(c, FileType(entry_format), err);
@@ -219,8 +216,7 @@ TrackingSyncSource::InsertItemResult MaemoCalendarSource::insertItem(const strin
 {
     int err;
     CComponent *c;
-    bool r;
-    InsertItemResultState u = ITEM_OKAY;
+    bool r, u = false;
     TrackingSyncSource::InsertItemResult result;
 
     if (cal->getCalendarType() == BIRTHDAY_CALENDAR) {
@@ -279,7 +275,7 @@ TrackingSyncSource::InsertItemResult MaemoCalendarSource::insertItem(const strin
             throwError(string("creating item "));
         }
         if (err == CALENDAR_ENTRY_DUPLICATED) {
-            u = ITEM_MERGED;
+            u = true;
         }
     }
 
@@ -292,12 +288,6 @@ TrackingSyncSource::InsertItemResult MaemoCalendarSource::insertItem(const strin
 void MaemoCalendarSource::removeItem(const string &uid)
 {
     int err;
-
-    if (cal->getCalendarType() == BIRTHDAY_CALENDAR) {
-        // stubbornly refuse to try this
-        throwError(string("can't sync smart calendar ") + cal->getCalendarName());
-    }
-
     cal->deleteComponent(uid, err);
     if (err != CALENDAR_OPERATION_SUCCESSFUL) {
         throwError(string("deleting item: ") + uid);
