@@ -42,11 +42,13 @@ class CalDAVSource : public WebDAVSource,
                                         const std::string &item);
     virtual void readSubItem(const std::string &uid, const std::string &subid, std::string &item);
     virtual std::string removeSubItem(const string &uid, const std::string &subid);
+    virtual void removeMergedItem(const std::string &luid);
     virtual void flushItem(const string &uid);
     virtual std::string getSubDescription(const string &uid, const string &subid);
     virtual void updateSynthesisInfo(SynthesisInfo &info,
                                      XMLConfigFragments &fragments) {
         info.m_backendRule = "HAVE-SYNCEVOLUTION-EXDATE-DETACHED";
+        info.m_globalIDs = true;
     }
 
     // implementation of SyncSourceLogging callback
@@ -76,8 +78,12 @@ class CalDAVSource : public WebDAVSource,
     virtual std::string serviceType() const { return "caldav"; }
     virtual bool typeMatches(const StringMap &props) const;
     virtual std::string homeSetProp() const { return "urn:ietf:params:xml:ns:caldav:calendar-home-set"; }
+    virtual std::string wellKnownURL() const { return "/.well-known/caldav"; }
+
     virtual std::string contentType() const { return "text/calendar; charset=utf-8"; }
     virtual std::string suffix() const { return ".ics"; }
+    virtual std::string getContent() const { return "VEVENT"; }
+    virtual bool getContentMixed() const { return true; }
 
  private:
     /**
@@ -174,6 +180,14 @@ class CalDAVSource : public WebDAVSource,
 
     std::string getSubDescription(Event &event, const string &subid);
 
+    /** calback for multiget: same as appendItem, but also records luid of all responses */
+    int appendMultigetResult(SubRevisionMap_t &revisions,
+                             std::set<std::string> &luids,
+                             const std::string &href,
+                             const std::string &etag,
+                             std::string &data);
+
+
     /** callback for listAllSubItems: parse and add new item */
     int appendItem(SubRevisionMap_t &revisions,
                    const std::string &href,
@@ -186,6 +200,12 @@ class CalDAVSource : public WebDAVSource,
                    const std::string &etag,
                    std::string &data);
 
+    /** callback for loadItem(): store right item from REPORT */
+    int storeItem(const std::string &wantedLuid,
+                  std::string &item,
+                  std::string &data,
+                  const std::string &href);
+
     /** add to m_cache */
     void addSubItem(const std::string &luid,
                     const SubRevisionEntry &entry);
