@@ -4,6 +4,7 @@
 
 #include "WebDAVSource.h"
 #include "CalDAVSource.h"
+#include "CalDAVVxxSource.h"
 #include "CardDAVSource.h"
 #include <syncevo/SyncSource.h>
 #ifdef ENABLE_UNIT_TESTS
@@ -42,11 +43,13 @@ static SyncSource *createSource(const SyncSourceParams &params)
             enabled = true;
         }
     }
-#else
+#elif defined(ENABLE_DAV)
     static bool enabled = true;
 #endif
 
-    isMe = sourceType.m_backend == "CalDAV";
+    isMe = sourceType.m_backend == "CalDAV" ||
+        sourceType.m_backend == "CalDAVTodo" ||
+        sourceType.m_backend == "CalDAVJournal";
     if (isMe) {
         if (sourceType.m_format == "" ||
             sourceType.m_format == "text/calendar" ||
@@ -55,11 +58,16 @@ static SyncSource *createSource(const SyncSourceParams &params)
 #ifdef ENABLE_DAV
             if (enabled) {
                 boost::shared_ptr<Neon::Settings> settings;
-                boost::shared_ptr<SubSyncSource> sub(new CalDAVSource(params, settings));
-                return new MapSyncSource(params, sub);
+                if (sourceType.m_backend == "CalDAV") {
+                    boost::shared_ptr<SubSyncSource> sub(new CalDAVSource(params, settings));
+                    return new MapSyncSource(params, sub);
+                } else {
+                    return new CalDAVVxxSource(sourceType.m_backend == "CalDAVTodo" ? "VTODO" : "VJOURNAL",
+                                               params, settings);
+                }
             }
 #endif
-            return RegisterSyncSource::InactiveSource;
+            return RegisterSyncSource::InactiveSource(params);
         }
     }
 
@@ -74,7 +82,7 @@ static SyncSource *createSource(const SyncSourceParams &params)
                 return new CardDAVSource(params, settings);
             }
 #endif
-            return RegisterSyncSource::InactiveSource;
+            return RegisterSyncSource::InactiveSource(params);
         }
     }
 
@@ -93,10 +101,18 @@ public:
 #endif
                            createSource,
                            "CalDAV\n"
+                           "   calendar events\n"
+                           "CalDAVTodo\n"
+                           "   tasks\n"
+                           "CalDAVJournal\n"
+                           "   memos\n"
                            "CardDAV\n"
+                           "   contacts\n"
                            ,
                            Values() +
                            Aliases("CalDAV")
+                           + Aliases("CalDAVTodo")
+                           + Aliases("CalDAVJournal")
                            + Aliases("CardDAV")
                            )
     {
@@ -204,12 +220,13 @@ namespace {
 class WebDAVTest : public RegisterSyncSourceTest {
     std::string m_server;
     std::string m_type;
+    std::string m_database;
     ConfigProps m_props;
 
 public:
     /**
      * @param server      for example, "yahoo", "google"
-     * @param type        "caldav" or "carddav"
+     * @param type        "caldav", "caldavtodo", "caldavjournal" or "carddav"
      * @param props       sync properties (username, password, syncURL, ...)
      *                    or key/value parameters for the testing (testcases)
      */
@@ -220,6 +237,8 @@ public:
                                props.get(type + "/testconfig",
                                          props.get("testconfig",
                                                    type == "caldav" ? "eds_event" :
+                                                   type == "caldavtodo" ? "eds_task" :
+                                                   type == "caldavjournal" ? "eds_memo" :
                                                    type == "carddav" ? "eds_contact" :
                                                    type))),
         m_server(server),
@@ -227,14 +246,17 @@ public:
         m_props(props)
     {}
 
+    std::string getDatabase() const { return m_database; }
+    void setDatabase(const std::string &database) { m_database = database; }
+
     virtual void updateConfig(ClientTestConfig &config) const
     {
         config.m_type = m_type.c_str();
         if (m_type == "caldav") {
             config.m_supportsReccurenceEXDates = true;
         }
-        config.m_createSourceA = boost::bind(&WebDAVTest::createSource, this, _3);
-        config.m_createSourceB = boost::bind(&WebDAVTest::createSource, this, _3);
+        config.m_createSourceA = boost::bind(&WebDAVTest::createSource, this, _2, _4);
+        config.m_createSourceB = boost::bind(&WebDAVTest::createSource, this, _2, _4);
         ConfigProps::const_iterator it = m_props.find(m_type + "/testcases");
         if (it != m_props.end() ||
             (it = m_props.find("testcases")) != m_props.end()) {
@@ -242,20 +264,66 @@ public:
         }
     }
 
-    TestingSyncSource *createSource(bool isSourceA) const
+    // This is very similar to client-test-app.cpp. TODO: refactor?!
+    TestingSyncSource *createSource(const std::string &clientID, bool isSourceA) const
     {
-        boost::shared_ptr<SyncConfig> context(new SyncConfig(string("target-config@client-test-") + m_server));
-        SyncSourceNodes nodes = context->getSyncSourceNodes(m_type,
-                                                            /* string("_") m_clientID + */
-                                                            string("_") + (isSourceA ? "A" : "B"));
+        std::string name = m_server + "_" + m_type;
+        const char *server = getenv("CLIENT_TEST_SERVER");
+        std::string config = "target-config@client-test";
+        if (server) {
+            config += "-";
+            config += server;
+        }
+        std::string tracking =
+            string("_") + clientID +
+            string("_") + (isSourceA ? "A" : "B");
 
-        // always set properties taken from the environment;
-        // TODO: "database" property (currently always uses the default)
-        nodes.getProperties()->setProperty("backend", m_type);
+        SE_LOG_DEBUG(NULL, NULL, "instantiating testing source %s in config %s, with tracking name %s",
+                     name.c_str(),
+                     config.c_str(),
+                     tracking.c_str());
+        boost::shared_ptr<SyncConfig> context(new SyncConfig(config));
+        SyncSourceNodes nodes = context->getSyncSourceNodes(name, tracking);
+
+        // Copy properties from the Client::Sync
+        // @<CLIENT_TEST_SERVER>_<clientID>/<name> config, to ensure
+        // that a testing source used as part of Client::Sync uses the
+        // same settings.
+        std::string peerName = std::string(server ? server : "no-such-server")  + "_" + clientID;
+        boost::shared_ptr<SyncConfig> peer(new SyncConfig(peerName));
+        SyncSourceNodes peerNodes = peer->getSyncSourceNodes(name);
+        SE_LOG_DEBUG(NULL, NULL, "overriding testing source %s properties with the ones from config %s = %s",
+                     name.c_str(),
+                     peerName.c_str(),
+                     peer->getRootPath().c_str());
+        BOOST_FOREACH(const ConfigProperty *prop, SyncSourceConfig::getRegistry()) {
+            if (prop->isHidden()) {
+                continue;
+            }
+            boost::shared_ptr<FilterConfigNode> node = peerNodes.getNode(*prop);
+            InitStateString value = prop->getProperty(*node);
+            SE_LOG_DEBUG(NULL, NULL, "   %s = %s (%s)",
+                         prop->getMainName().c_str(),
+                         value.c_str(),
+                         value.wasSet() ? "set" : "default");
+            node = nodes.getNode(*prop);
+            node->setProperty(prop->getMainName(), value);
+        }
+        // Also copy loglevel.
+        context->setLogLevel(peer->getLogLevel());
+        context->flush();
+
+
+        // Always set properties taken from the environment.
+        nodes.getProperties()->setProperty("backend", InitStateString(m_type, true));
+        SE_LOG_DEBUG(NULL, NULL, "   additional property backend = %s (from CLIENT_TEST_WEBDAV)",
+                     m_type.c_str());
         BOOST_FOREACH(const StringPair &propval, m_props) {
             boost::shared_ptr<FilterConfigNode> node = context->getNode(propval.first);
             if (node) {
-                node->setProperty(propval.first, propval.second);
+                SE_LOG_DEBUG(NULL, NULL, "   additional property %s = %s (from CLIENT_TEST_WEBDAV)",
+                             propval.first.c_str(), propval.second.c_str());
+                node->setProperty(propval.first, InitStateString(propval.second, true));
             } else if (!boost::ends_with(propval.first, "testconfig") &&
                        !boost::ends_with(propval.first, "testcases")) {
                 SE_THROW(StringPrintf("invalid property %s=%s set in CLIENT_TEST_WEBDAV for %s %s",
@@ -269,6 +337,8 @@ public:
                                 nodes,
                                 context);
         SyncSource *ss = SyncSource::createSource(params);
+        ss->setDisplayName(ss->getDisplayName() +
+                           (isSourceA ? " #A" : " #B"));
         return static_cast<TestingSyncSource *>(ss);
     }
 };
@@ -278,12 +348,52 @@ public:
  * creates WebDAV sources by parsing
  * CLIENT_TEST_WEBDAV=<server> [caldav] [carddav] <prop>=<val> ...; ...
  */
-static class WebDAVTestSingleton {
-    list< boost::shared_ptr<WebDAVTest> > m_sources;
+static class WebDAVTestSingleton : RegisterSyncSourceTest {
+    /**
+     * It could be that different sources are configured to use
+     * the same resource (= database property). Get the database
+     * property of each source by instantiating it. Check against
+     * already added entries and if a match is found, record the
+     * link. This enables the Client::Source::xxx::testLinkedSources
+     * test of that previous entry.
+     */
+    class WebDAVList
+    {
+        list< boost::shared_ptr<WebDAVTest> >m_sources;
+
+    public:
+        void push_back(const boost::shared_ptr<WebDAVTest> &source)
+        {
+            boost::scoped_ptr<TestingSyncSource> instance(source->createSource("1", true));
+            std::string database = instance->getDatabaseID();
+            source->setDatabase(database);
+
+            BOOST_FOREACH (const boost::shared_ptr<WebDAVTest> &other,
+                           m_sources) {
+                if (other->getDatabase() == database) {
+                    other->m_linkedSources.push_back(source->m_configName);
+                    break;
+                }
+            }
+            m_sources.push_back(source);
+        }
+    };
+    mutable WebDAVList m_sources;
 
 public:
-    WebDAVTestSingleton()
+    WebDAVTestSingleton() :
+        RegisterSyncSourceTest("", "") // empty, only purpose is to get init() called
+    {}
+
+    virtual void updateConfig(ClientTestConfig &config) const {}
+    virtual void init() const
     {
+        static bool initialized;
+        if (initialized) {
+            return;
+        }
+        initialized = true;
+
         const char *env = getenv("CLIENT_TEST_WEBDAV");
         if (!env) {
             return;
@@ -295,7 +405,10 @@ public:
         BOOST_FOREACH(const std::string &entry,
                       boost::tokenizer< boost::char_separator<char> >(settings, boost::char_separator<char>(";"))) {
             std::string server;
-            bool caldav = false, carddav = false;
+            bool caldav = false,
+                caldavtodo = false,
+                caldavjournal = false,
+                carddav = false;
             ConfigProps props;
             BOOST_FOREACH(const std::string &token,
                           boost::tokenizer< boost::char_separator<char> >(entry, boost::char_separator<char>("\t "))) {
@@ -303,6 +416,10 @@ public:
                     server = token;
                 } else if (token == "caldav") {
                     caldav = true;
+                } else if (token == "caldavtodo") {
+                    caldavtodo = true;
+                } else if (token == "caldavjournal") {
+                    caldavjournal = true;
                 } else if (token == "carddav") {
                     carddav = true;
                 } else {
@@ -313,10 +430,19 @@ public:
                     props[token.substr(0,pos)] = token.substr(pos + 1);
                 }
             }
+
             if (caldav) {
                 boost::shared_ptr<WebDAVTest> ptr(new WebDAVTest(server, "caldav", props));
                 m_sources.push_back(ptr);
             }
+            if (caldavtodo) {
+                boost::shared_ptr<WebDAVTest> ptr(new WebDAVTest(server, "caldavtodo", props));
+                m_sources.push_back(ptr);
+            }
+            if (caldavjournal) {
+                boost::shared_ptr<WebDAVTest> ptr(new WebDAVTest(server, "caldavjournal", props));
+                m_sources.push_back(ptr);
+            }
             if (carddav) {
                 boost::shared_ptr<WebDAVTest> ptr(new WebDAVTest(server, "carddav", props));
                 m_sources.push_back(ptr);
