@@ -22,8 +22,20 @@
 #define INCL_LOGGING
 
 #include <stdarg.h>
+#include <stdio.h>
 #include <string>
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+#ifdef HAVE_GLIB
+# include <glib.h>
+#endif
+
+#include <syncevo/Timespec.h>
+
+#include <boost/function.hpp>
+
 #include <syncevo/declarations.h>
 SE_BEGIN_CXX
 
@@ -107,7 +119,31 @@ class Logger
      */
     static void setProcessName(const std::string &name) { m_processName = name; }
     static std::string getProcessName() { return m_processName; }
-    
+
+#ifdef HAVE_GLIB
+    /**
+     * can be used in g_log_set_handler() to redirect log messages
+     * into our own logging; must be called for each log domain
+     * that may be relevant
+     */
+    static void glogFunc(const gchar *logDomain,
+                         GLogLevelFlags logLevel,
+                         const gchar *message,
+                         gpointer userData);
+#endif
+
+    /**
+     * can be used as replacement for libsynthesis console printf function,
+     * logs at DEBUG level
+     *
+     * @param stream   is ignored
+     * @param format   guaranteed to start with "SYSYNC "
+     * @return always 0
+     */
+    static int sysyncPrintf(FILE *stream,
+                            const char *format,
+                            ...);
+
     virtual ~Logger() {}
 
     /**
@@ -200,8 +236,34 @@ class LoggerBase : public Logger
     virtual void setLevel(Level level) { m_level = level; }
     virtual Level getLevel() { return m_level; }
 
+ protected:
+    /**
+     * Prepares the output. The result is passed back to the caller
+     * line-by-line (expectedTotal > 0) and/or as full chunk
+     * (expectedTotal = 0). The expected size is just a guess, be
+     * prepared to handle more output.
+     *
+     * Each chunk already includes the necessary line breaks (in
+     * particular after the last line when it contains the entire
+     * output). It may be modified by the callback.
+     */
+    void formatLines(Level msglevel,
+                     Level outputlevel,
+                     const std::string &processName,
+                     const char *prefix,
+                     const char *format,
+                     va_list args,
+                     boost::function<void (std::string &chunk, size_t expectedTotal)> print);
+
  private:
     Level m_level;
+
+    /**
+     * Set by formatLines() before writing the first message if log
+     * level is debugging, together with printing a message that gives
+     * the local time.
+     */
+    Timespec m_startTime;
 };
 
 
