@@ -213,27 +213,31 @@ class SingleIniTest : public CppUnit::TestFixture {
         CPPUNIT_ASSERT(node);
         CPPUNIT_ASSERT(node->exists());
         CPPUNIT_ASSERT_EQUAL(string("test - /foo/config.ini"), node->getName());
-        CPPUNIT_ASSERT_EQUAL(string("bar"), node->readProperty("foo"));
-        CPPUNIT_ASSERT_EQUAL(string("bar2"), node->readProperty("foo2"));
+        CPPUNIT_ASSERT(node->readProperty("foo").wasSet());
+        CPPUNIT_ASSERT_EQUAL(string("bar"), node->readProperty("foo").get());
+        CPPUNIT_ASSERT_EQUAL(string("bar2"), node->readProperty("foo2").get());
+        CPPUNIT_ASSERT(node->readProperty("foo2").wasSet());
+        CPPUNIT_ASSERT_EQUAL(string(""), node->readProperty("no_such_bar").get());
+        CPPUNIT_ASSERT(!node->readProperty("no_such_bar").wasSet());
         node = tree.open("/foo/config.ini");
         CPPUNIT_ASSERT(node);
         CPPUNIT_ASSERT(node->exists());
         node = tree.open("foo//.config.ini");
         CPPUNIT_ASSERT(node);
         CPPUNIT_ASSERT(node->exists());
-        CPPUNIT_ASSERT_EQUAL(string("bar_internal"), node->readProperty("foo_internal"));
-        CPPUNIT_ASSERT_EQUAL(string("bar2_internal"), node->readProperty("foo2_internal"));
+        CPPUNIT_ASSERT_EQUAL(string("bar_internal"), node->readProperty("foo_internal").get());
+        CPPUNIT_ASSERT_EQUAL(string("bar2_internal"), node->readProperty("foo2_internal").get());
         node = tree.open("bar///./.internal.ini");
         CPPUNIT_ASSERT(node);
         CPPUNIT_ASSERT(node->exists());
-        CPPUNIT_ASSERT_EQUAL(string("foo"), node->readProperty("bar"));
+        CPPUNIT_ASSERT_EQUAL(string("foo"), node->readProperty("bar").get());
         node = tree.open("sources/addressbook/config.ini");
         CPPUNIT_ASSERT(node);
         CPPUNIT_ASSERT(node->exists());
         node = tree.open("sources/calendar/config.ini");
         CPPUNIT_ASSERT(node);
         CPPUNIT_ASSERT(node->exists());
-        CPPUNIT_ASSERT_EQUAL(string("Personal"), node->readProperty("evolutionsource"));
+        CPPUNIT_ASSERT_EQUAL(string("Personal"), node->readProperty("evolutionsource").get());
 
         node = tree.open("no-such-source/config.ini");
         CPPUNIT_ASSERT(node);
