@@ -27,7 +27,7 @@
 #include <stdarg.h>
 
 #include <syncevo/declarations.h>
-
+using namespace std;
 namespace {
 
 std::string lookupDebug, lookupInfo;
@@ -184,7 +184,7 @@ extern "C" void EDSAbiWrapperInit()
 #ifdef EVOLUTION_COMPATIBILITY
 # ifdef HAVE_EDS
     edshandle =
-    findSymbols("libedataserver-1.2.so", 7, 14,
+    findSymbols("libedataserver-1.2.so", 7, 16,
                 FIND_SYMBOLS_NEED_ALL|FIND_SYMBOLS_LENIENT_MAX_VERSION, NULL,
                 &EDSAbiWrapperSingleton.e_source_get_type, "e_source_get_type",
                 &EDSAbiWrapperSingleton.e_source_get_uri, "e_source_get_uri",
@@ -197,8 +197,10 @@ extern "C" void EDSAbiWrapperInit()
 # endif // HAVE_EDS
 
 # ifdef ENABLE_EBOOK
+    static const int libebookMinVersion = 5,
+        libebookMaxVersion = 13;
     ebookhandle =
-    findSymbols("libebook-1.2.so", 5, 12,
+    findSymbols("libebook-1.2.so", libebookMinVersion, libebookMaxVersion,
                 FIND_SYMBOLS_NEED_ALL|FIND_SYMBOLS_LENIENT_MAX_VERSION, NULL,
                 &EDSAbiWrapperSingleton.e_book_add_contact, "e_book_add_contact",
                 &EDSAbiWrapperSingleton.e_book_authenticate_user, "e_book_authenticate_user",
@@ -228,11 +230,16 @@ extern "C" void EDSAbiWrapperInit()
                 &EDSAbiWrapperSingleton.e_vcard_to_string, "e_vcard_to_string",
                 (void *)0);
     EDSAbiHaveEbook = EDSAbiWrapperSingleton.e_book_new != 0;
+    findSymbols("libebook-1.2.so", libebookMinVersion, libebookMaxVersion,
+                FIND_SYMBOLS_LENIENT_MAX_VERSION, NULL,
+                &EDSAbiWrapperSingleton.e_contact_inline_local_photos, "e_contact_inline_local_photos",
+                (void *)0);
+
 # endif // ENABLE_EBOOK
 
 # ifdef ENABLE_ECAL
     static const int libecalMinVersion = 3,
-        libecalMaxVersion = 10;
+        libecalMaxVersion = 11;
     ecalhandle =
     findSymbols("libecal-1.2.so", libecalMinVersion, libecalMaxVersion,
                 FIND_SYMBOLS_NEED_ALL|FIND_SYMBOLS_LENIENT_MAX_VERSION, NULL,
