@@ -34,6 +34,7 @@
 #include <SyncSource.h>
 
 #include "test.h"
+#include "ClientTestAssert.h"
 
 #ifdef ENABLE_INTEGRATION_TESTS
 
@@ -41,6 +42,9 @@
 #include <cppunit/TestAssert.h>
 #include <cppunit/TestFixture.h>
 
+#include <syncevo/Logging.h>
+#include <syncevo/util.h>
+
 #include <syncevo/declarations.h>
 SE_BEGIN_CXX
 
@@ -69,15 +73,15 @@ class CheckSyncReport {
         serverAdded(srAdded),
         serverUpdated(srUpdated),
         serverDeleted(srDeleted),
+        restarts(0),
         mustSucceed(mstSucceed),
         syncMode(mode),
         m_report(NULL)
         {}
 
-    virtual ~CheckSyncReport() {}
-
     int clientAdded, clientUpdated, clientDeleted,
         serverAdded, serverUpdated, serverDeleted;
+    int restarts;
     bool mustSucceed;
     SyncMode syncMode;
 
@@ -86,7 +90,7 @@ class CheckSyncReport {
 
     CheckSyncReport &setMode(SyncMode mode) { syncMode = mode; return *this; }
     CheckSyncReport &setReport(SyncReport *report) { m_report = report; return *this; }
-
+    CheckSyncReport &setRestarts(int r) { restarts = r; return *this; }
 
     /**
      * checks that the sync completed as expected and throws
@@ -95,7 +99,12 @@ class CheckSyncReport {
      * @param res     return code from SyncClient::sync()
      * @param report  the sync report stored in the SyncClient
      */
-    virtual void check(SyncMLStatus status, SyncReport &report) const;
+    void check(SyncMLStatus status, SyncReport &report) const;
+
+    /**
+     * checks that the source report matches with expectations
+     */
+    void check(const std::string &name, const SyncSourceReport &report) const;
 };
 
 /**
@@ -131,13 +140,13 @@ struct SyncOptions {
     
     bool m_isAborted;
 
-    typedef boost::function<bool (SyncContext &,
-                                  SyncOptions &)> Callback_t;
     /**
      * Callback to be invoked after setting up local sources, but
      * before running the engine. May throw exception to indicate
      * error and return true to stop sync without error.
      */
+    typedef boost::function<bool (SyncContext &,
+                                  SyncOptions &)> Callback_t;
     Callback_t m_startCallback;
 
     /**
@@ -232,6 +241,9 @@ class ClientTest {
     ClientTest(int serverSleepSec = 0, const std::string &serverLog= "");
     virtual ~ClientTest();
 
+    /** a unique string - "1" or "2" in practice */
+    virtual std::string getClientID() const = 0;
+
     /** set up before running a test */
     virtual void setup() { }
 
@@ -298,11 +310,13 @@ class ClientTest {
     static void getItems(const std::string &file, std::list<std::string> &items, std::string &realfile);
 
     /**
-     * utility function for importing items with blank lines as separator
+     * utility function for importing items with blank lines as separator,
+     * for ClientTestConfig::m_import
      */
     static std::string import(ClientTest &client, TestingSyncSource &source,
                               const ClientTestConfig &config,
-                              const std::string &file, std::string &realfile);
+                              const std::string &file, std::string &realfile,
+                              std::list<std::string> *luids);
 
     /**
      * utility function for comparing vCard and iCal files with the external
@@ -437,7 +451,7 @@ public:
 
     TestingSyncSource *operator() () {
         CPPUNIT_ASSERT(createSource);
-        return createSource(client, source, isSourceA);
+        return createSource(client, client.getClientID(), source, isSourceA);
     }
 
     const ClientTest::Config::createsource_t createSource;
@@ -462,6 +476,19 @@ public:
     /** configuration that corresponds to source */
     const ClientTest::Config config;
 
+    /** shortcut for config.m_sourceName */
+    const std::string &getSourceName() const { return config.m_sourceName; }
+
+    /**
+     * A list of config pointers which share the same
+     * database. Normally, sources are tested in isolation, but for
+     * such linked sources we also need to test interdependencies, in
+     * particular regarding change tracking and item listing.
+     *
+     * This includes *all* configs, not just the other ones.
+     */
+    std::list<LocalTests *> m_linkedSources;
+
     /** helper funclets to create sources */
     CreateSource createSourceA, createSourceB;
 
@@ -525,6 +552,12 @@ public:
      */
     virtual bool compareDatabases(const char *refFile, TestingSyncSource &copy, bool raiseAssert = true);
 
+    /**
+     * compare data in source with set of items
+     */
+    void compareDatabasesRef(TestingSyncSource &copy,
+                             const std::list<std::string> &items);
+
     /**
      * compare data in source with vararg list of std::string pointers, NULL terminated
      */
@@ -543,6 +576,26 @@ public:
     virtual std::list<std::string> insertManyItems(CreateSource createSource, int startIndex = 1, int numItems = 0, int size = -1);
     virtual std::list<std::string> insertManyItems(TestingSyncSource *source, int startIndex = 1, int numItems = 0, int size = -1);
 
+    /**
+     * Update existing items. Must match a corresponding previous call to
+     * insertManyItems().
+     *
+     * @param revision    revision number, used to distinguish different generations of each item
+     * @param luids       result from corresponding insertManyItems() call
+     * @param offset      skip that many items at the start of luids before updating the following ones
+     */
+    void updateManyItems(CreateSource createSource, int startIndex, int numItems, int size,
+                         int revision,
+                         std::list<std::string> &luids,
+                         int offset);
+
+    /**
+     * Delete items. Skips offset items in luids before deleting numItems.
+     */
+    void removeManyItems(CreateSource createSource, int numItems,
+                         std::list<std::string> &luids,
+                         int offset);
+
     /**
      * update every single item, using config.update
      */
@@ -574,9 +627,13 @@ public:
     virtual void testLocalDeleteAll();
     virtual void testComplexInsert();
     virtual void testLocalUpdate();
+    void doChanges(bool restart);
     virtual void testChanges();
+    virtual void testChangesMultiCycles();
+    virtual void testLinkedSources();
     virtual void testImport();
     virtual void testImportDelete();
+    virtual void testRemoveProperties();
     virtual void testManyChanges();
     virtual void testLinkedItemsParent();
     virtual void testLinkedItemsChild();
@@ -591,14 +648,17 @@ public:
     virtual void testLinkedItemsUpdateChild();
     virtual void testLinkedItemsInsertBothUpdateChild();
     virtual void testLinkedItemsInsertBothUpdateParent();
+    virtual void testLinkedItemsInsertBothUpdateChildNoIDs();
+    virtual void testLinkedItemsUpdateChildNoIDs();
     virtual void testLinkedItemsSingle404();
     virtual void testLinkedItemsMany404();
 
+    virtual void testSubset();
+
     /** retrieve right set of items for running test */
     ClientTestConfig::LinkedItems_t getParentChildData();
 };
 
-int countItemsOfType(TestingSyncSource *source, int state);
 std::list<std::string> listItemsOfType(TestingSyncSource *source, int state);
 
 /**
@@ -688,52 +748,42 @@ protected:
 
     /* for more information on the different tests see their implementation */
 
-    // do a two-way sync without additional checks,
-    // may or may not actually be done in two-way mode
-    virtual void testTwoWaySync() {
-        doSync(SyncOptions(SYNC_TWO_WAY));
-    }
+    virtual void testTwoWaySync();
+    virtual void testSlowSync();
+    virtual void testRefreshFromServerSync();
+    virtual void testRefreshFromClientSync();
+    virtual void testRefreshFromRemoteSync();
+    virtual void testRefreshFromLocalSync();
 
-    // do a slow sync without additional checks
-    virtual void testSlowSync() {
-        doSync(SyncOptions(SYNC_SLOW,
-                           CheckSyncReport(-1,-1,-1, -1,-1,-1, true, SYNC_SLOW)));
-    }
-    // do a refresh from server sync without additional checks
-    virtual void testRefreshFromServerSync() {
-        doSync(SyncOptions(SYNC_REFRESH_FROM_SERVER,
-                           CheckSyncReport(-1,-1,-1, -1,-1,-1, true, SYNC_REFRESH_FROM_SERVER)));
-    }
-
-    // do a refresh from client sync without additional checks
-    virtual void testRefreshFromClientSync() {
-        doSync(SyncOptions(SYNC_REFRESH_FROM_CLIENT,
-                           CheckSyncReport(-1,-1,-1, -1,-1,-1, true, SYNC_REFRESH_FROM_CLIENT)));
-    }
-
-    // delete all items, locally and on server using two-way sync
-    virtual void testDeleteAllSync() {
-        deleteAll(DELETE_ALL_SYNC);
-    }
+    virtual void testDeleteAllSync();
 
     virtual void testDeleteAllRefresh();
     virtual void testRefreshFromClientSemantic();
     virtual void testRefreshFromServerSemantic();
     virtual void testRefreshStatus();
 
-    // test that a two-way sync copies an item from one address book into the other
-    void testCopy() {
-        doCopy();
-        compareDatabases();
-    }
+    void doRestartSync(SyncMode mode);
+    void testTwoWayRestart();
+    void testSlowRestart();
+    void testRefreshFromLocalRestart();
+    void testOneWayFromLocalRestart();
+    void testRefreshFromRemoteRestart();
+    void testOneWayFromRemoteRestart();
+    void testManyRestarts();
+
+    void testCopy();
 
     virtual void testUpdate();
     virtual void testComplexUpdate();
     virtual void testDelete();
     virtual void testMerge();
     virtual void testTwinning();
-    virtual void testOneWayFromServer();
-    virtual void testOneWayFromClient();
+    void doOneWayFromRemote(SyncMode oneWayFromRemote);
+    void testOneWayFromServer();
+    void testOneWayFromRemote();
+    void doOneWayFromLocal(SyncMode oneWayFromLocal);
+    void testOneWayFromClient();
+    void testOneWayFromLocal();
     bool doConversionCallback(bool *success,
                               SyncContext &client,
                               SyncOptions &options);
@@ -743,14 +793,8 @@ protected:
     virtual void testExtensions();
     virtual void testAddUpdate();
 
-    // test copying with maxMsg and no large object support
-    void testMaxMsg() {
-        doVarSizes(true, false);
-    }
-    // test copying with maxMsg and large object support
-    void testLargeObject() {
-        doVarSizes(true, true);
-    }
+    void testMaxMsg();
+    void testLargeObject();
 
     virtual void testManyItems();
     virtual void testManyDeletes();
@@ -844,7 +888,30 @@ protected:
         SyncPrefix prefix(logPrefix, *this);
         doSync(options);
     }
+    void doSync(const char *file, int line,
+                const char *logPrefix,
+                const SyncOptions &options) {
+        CT_WRAP_ASSERT(file, line, doSync(logPrefix, options));
+    }
+    void doSync(const char *file, int line,
+                const SyncOptions &options) {
+        CT_WRAP_ASSERT(file, line, doSync(options));
+    }
     virtual void postSync(int res, const std::string &logname);
+
+ private:
+    void allSourcesInsert();
+    void allSourcesUpdate();
+    void allSourcesDeleteAll();
+    void allSourcesInsertMany(int startIndex, int numItems,
+                              std::map<int, std::list<std::string> > &luids);
+    void allSourcesUpdateMany(int startIndex, int numItems,
+                              int revision,
+                              std::map<int, std::list<std::string> > &luids,
+                              int offset);
+    void allSourcesRemoveMany(int numItems,
+                              std::map<int, std::list<std::string> > &luids,
+                              int offset);
 };
 
 /*
@@ -899,40 +966,46 @@ public:
     virtual void setTimeout(int seconds) { m_wrappedAgent->setTimeout(seconds); }
 };
 
+/** write log message into *.log file of a test */
+#define CLIENT_TEST_LOG(_format, _args...) \
+    SE_LOG_DEBUG(NULL, NULL, "\n%s:%d *** " _format, \
+                 getBasename(__FILE__).c_str(), __LINE__, \
+                 ##_args)
+
 /** assert equality, include string in message if unequal */
 #define CLIENT_TEST_EQUAL( _prefix, \
                            _expected, \
                            _actual ) \
-    CPPUNIT_ASSERT_EQUAL_MESSAGE( std::string(_prefix) + ": " + #_expected + " == " + #_actual, \
-                                  _expected, \
-                                  _actual )
+    CT_ASSERT_EQUAL_MESSAGE( std::string(_prefix) + ": " + #_expected + " == " + #_actual, \
+                             _expected, \
+                             _actual )
 
 /** execute _x and then check the status of the _source pointer */
 #define SOURCE_ASSERT_NO_FAILURE(_source, _x) \
 { \
-    CPPUNIT_ASSERT_NO_THROW(_x); \
-    CPPUNIT_ASSERT((_source)); \
+    CT_ASSERT_NO_THROW(_x); \
+    CT_ASSERT((_source)); \
 }
 
 /** check _x for true and then the status of the _source pointer */
 #define SOURCE_ASSERT(_source, _x) \
 { \
-    CPPUNIT_ASSERT(_x); \
-    CPPUNIT_ASSERT((_source)); \
+    CT_ASSERT(_x); \
+    CT_ASSERT((_source)); \
 }
 
 /** check that _x evaluates to a specific value and then the status of the _source pointer */
 #define SOURCE_ASSERT_EQUAL(_source, _value, _x) \
 { \
-    CPPUNIT_ASSERT_EQUAL(_value, _x); \
-    CPPUNIT_ASSERT((_source)); \
+    CT_ASSERT_EQUAL(_value, _x); \
+    CT_ASSERT((_source)); \
 }
 
 /** same as SOURCE_ASSERT() with a specific failure message */
 #define SOURCE_ASSERT_MESSAGE(_message, _source, _x)     \
 { \
-    CPPUNIT_ASSERT_MESSAGE((_message), (_x)); \
-    CPPUNIT_ASSERT((_source)); \
+    CT_ASSERT_MESSAGE((_message), (_x)); \
+    CT_ASSERT((_source)); \
 }
 
 /**
@@ -948,6 +1021,9 @@ public:
 #define ADD_TEST_TO_SUITE(_suite, _class, _function) \
     _suite->addTest(FilterTest(new CppUnit::TestCaller<_class>(_suite->getName() + "::" #_function, &_class::_function, *this)))
 
+#define ADD_TEST_TO_SUITE_SUFFIX(_suite, _class, _function, _suffix) \
+    _suite->addTest(FilterTest(new CppUnit::TestCaller<_class>(_suite->getName() + "::" #_function + _suffix, &_class::_function, *this)))
+
 SE_END_CXX
 
 #endif // ENABLE_INTEGRATION_TESTS
