@@ -29,6 +29,14 @@
 
 #include "test.h"
 
+#include <signal.h>
+#ifdef HAVE_VALGRIND_VALGRIND_H
+# include <valgrind/valgrind.h>
+#endif
+#ifdef HAVE_EXECINFO_H
+# include <execinfo.h>
+#endif
+
 #include <cppunit/CompilerOutputter.h>
 #include <cppunit/ui/text/TestRunner.h>
 #include <cppunit/TestListener.h>
@@ -41,8 +49,13 @@
 #include <Logging.h>
 #include <LogStdout.h>
 #include <syncevo/LogRedirect.h>
+#include <syncevo/SyncContext.h>
 #include "ClientTest.h"
 
+#include <boost/algorithm/string/split.hpp>
+
+#include <pcrecpp.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -50,6 +63,10 @@
 # include <signal.h>
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #include <string>
 #include <stdexcept>
 
@@ -112,17 +129,7 @@ public:
     }
 
     void addAllowedFailures(string allowedFailures) {
-        size_t start = 0, end;
-        while ((end = allowedFailures.find(',', start)) != allowedFailures.npos) {
-            size_t len = end - start;
-            if (len) {
-                m_allowedFailures.insert(allowedFailures.substr(start, len));
-            }
-            start = end + 1;
-        }
-        if (allowedFailures.size() > start) {
-            m_allowedFailures.insert(allowedFailures.substr(start));
-        }
+        boost::split(m_allowedFailures, allowedFailures, boost::is_from_range(',', ','));
     }
 
     void startTest (CppUnit::Test *test) {
@@ -165,11 +172,16 @@ public:
             CppUnit::CompilerOutputter formatter(&m_failures, output);
             formatter.printFailureReport();
             failure = output.str();
-            if (m_allowedFailures.find(m_currentTest) == m_allowedFailures.end()) {
+            m_failed = true;
+            BOOST_FOREACH (const std::string &re, m_allowedFailures) {
+                if (pcrecpp::RE(re).FullMatch(m_currentTest)) {
+                    result = "*** failure ignored ***";
+                    m_failed = false;
+                    break;
+                }
+            }
+            if (m_failed) {
                 result = "*** failed ***";
-                m_failed = true;
-            } else {
-                result = "*** failure ignored ***";
             }
         } else {
             result = "okay";
@@ -177,7 +189,7 @@ public:
 
         SE_LOG_DEBUG(NULL, NULL, "*** ending %s: %s ***", m_currentTest.c_str(), result.c_str());
         if (!failure.empty()) {
-            SE_LOG_DEBUG(NULL, NULL, "%s", failure.c_str());
+            SE_LOG_ERROR(NULL, NULL, "%s", failure.c_str());
         }
         if (&LoggerBase::instance() == m_logger.get()) {
             LoggerBase::popLogger();
@@ -189,13 +201,31 @@ public:
         
         const char* compareLog = getenv("CLIENT_TEST_COMPARE_LOG");
         if(compareLog && strlen(compareLog)) {
-            FILE *fd = fopen ("____compare.log","r");
-            if (fd != NULL) {
-                fclose(fd);
-                if (system ((string("cat ____compare.log >>")+logfile).c_str()) < 0) {
-                    SE_LOG_WARNING(NULL, NULL, "Unable to append ____compare.log to %s.",
-                                   logfile.c_str());
+            int fd = open("____compare.log", O_RDONLY);
+            if (fd >= 0) {
+                int out = open(logfile.c_str(), O_WRONLY|O_APPEND);
+                if (out >= 0) {
+                    char buffer[4096];
+                    bool cont = true;
+                    ssize_t len;
+                    while (cont && (len = read(fd, buffer, sizeof(buffer))) > 0) {
+                        ssize_t total = 0;
+                        while (cont && total < len) {
+                            ssize_t written = write(out, buffer, len);
+                            if (written < 0) {
+                                perror(("writing " + logfile).c_str());
+                                cont = false;
+                            } else {
+                                total += written;
+                            }
+                        }
+                    }
+                    if (len < 0) {
+                        perror("reading ____compare.log");
+                    }
+                    close(out);
                 }
+                close(fd);
             }
         }
 
@@ -239,9 +269,41 @@ static void printTests(CppUnit::Test *test, int indention)
     }
 }
 
+static void handler(int sig)
+{
+    void *buffer[100];
+    int size;
+
+    fprintf(stderr, "\ncaught signal %d\n", sig);
+    fflush(stderr);
+#ifdef HAVE_EXECINFO_H
+    size = backtrace(buffer, sizeof(buffer)/sizeof(buffer[0]));
+    backtrace_symbols_fd(buffer, size, 2);
+#endif
+#ifdef HAVE_VALGRIND_VALGRIND_H
+    VALGRIND_PRINTF_BACKTRACE("\ncaught signal %d\n", sig);
+#endif
+    /* system("objdump -l -C -d client-test >&2"); */
+    struct sigaction act;
+    memset(&act, 0, sizeof(act));
+    act.sa_handler = SIG_DFL;
+    sigaction(SIGABRT, &act, NULL);
+    abort();
+}
+
 extern "C"
 int main(int argc, char* argv[])
 {
+  SyncContext::initMain("client-test");
+
+  struct sigaction act;
+
+  memset(&act, 0, sizeof(act));
+  act.sa_handler = handler;
+  sigaction(SIGABRT, &act, NULL);
+  sigaction(SIGSEGV, &act, NULL);
+  sigaction(SIGILL, &act, NULL);
+
   // Get the top level suite from the registry
   CppUnit::Test *suite = CppUnit::TestFactoryRegistry::getRegistry().makeTest();
 
