@@ -16,7 +16,11 @@ export DBUS_SESSION_BUS_ADDRESS
 KEYRING_PID=$!
 
 # kill all programs started by us
-trap "kill $KEYRING_PID; kill $DBUS_SESSION_BUS_PID" EXIT
+atexit() {
+    kill $KEYRING_PID
+    kill $DBUS_SESSION_BUS_PID
+}
+trap atexit EXIT
 
 # If DBUS_SESSION_SH_EDS_BASE is set and our main program runs
 # under valgrind, then also check EDS. Otherwise start EDS only
@@ -25,24 +29,43 @@ trap "kill $KEYRING_PID; kill $DBUS_SESSION_BUS_PID" EXIT
 #
 # DBUS_SESSION_SH_EDS_BASE must be the directory which contains
 # e-addressbook/calendar-factory.
+#
+# Similar for DBUS_SESSION_SH_AKONADI, except that we rely on akonadictl
+# and don't run it under valgrind. We also don't start it for test-dbus.py,
+# because starting it with "normal" XDG env variables and trying to
+# connect to it with XDG env set in test-dbus.py failed:
+# libakonadi Akonadi::SessionPrivate::init: Akonadi Client Session: connection config file ' akonadi/akonadiconnectionrc can not be found in ' "/data/runtests/work/lucid-amd64/build/src/temp-test-dbus/config" ' nor in any of  ("/etc/xdg", "/etc")
+#
 E_CAL_PID=
 E_BOOK_PID=
 case "$@" in *valgrind*) prefix=`echo $@ | perl -p -e 's;.*?(\S*/?valgrind\S*).*;$1;'`;;
-             *syncevolution\ *|*client-test\ *|*test-dbus.py\ *) prefix=env;;
+             *syncevolution\ *|*client-test\ *|*bash*|*test-dbus.py\ *|*gdb\ *) prefix=env;;
              *) prefix=;; # don't start EDS
 esac
+akonadi=$prefix
+case "$@" in *test-dbus.py\ *) akonadi=;;
+esac
+
+if [ "$DBUS_SESSION_SH_AKONADI" ] && [ "$akonadi" ]; then
+    akonadictl start
+    SLEEP=5
+else
+    DBUS_SESSION_SH_AKONADI=
+fi
 if [ "$DBUS_SESSION_SH_EDS_BASE" ] && [ "$prefix" ]; then
-    $prefix $DBUS_SESSION_SH_EDS_BASE/e-calendar-factory --keep-running &
+    $prefix $DBUS_SESSION_SH_EDS_BASE/evolution-calendar-factory --keep-running &
     E_CAL_PID=$!
-    $prefix $DBUS_SESSION_SH_EDS_BASE/e-addressbook-factory --keep-running &
+    $prefix $DBUS_SESSION_SH_EDS_BASE/evolution-addressbook-factory --keep-running &
     E_BOOK_PID=$!
 
     # give daemons some time to start and register with D-Bus
-    sleep 5
+    SLEEP=5
 else
     DBUS_SESSION_SH_EDS_BASE=
 fi
 
+sleep $SLEEP
+
 # run program
 "$@"
 res=$?
@@ -74,11 +97,15 @@ shutdown () {
     esac
 }
 
+if [ "$DBUS_SESSION_SH_AKONADI" ]; then
+    akonadictl stop
+fi
+
 if [ "$DBUS_SESSION_SH_EDS_BASE" ]; then
     kill "$E_CAL_PID" 2>/dev/null
     kill "$E_BOOK_PID" 2>/dev/null
-    shutdown "$E_CAL_PID" "$DBUS_SESSION_SH_EDS_BASE/e-calendar-factory"
-    shutdown "$E_BOOK_PID" "$DBUS_SESSION_SH_EDS_BASE/e-addressbook-factory"
+    shutdown "$E_CAL_PID" "$DBUS_SESSION_SH_EDS_BASE/evolution-calendar-factory"
+    shutdown "$E_BOOK_PID" "$DBUS_SESSION_SH_EDS_BASE/evolution-addressbook-factory"
 fi
 
 echo dbus-session.sh: final result $res >&2
